from qdrant_client.http.exceptions import ApiException as QdrantAPIException

from unstract.sdk.adapters.vectordb.qdrant.src import Qdrant
from unstract.sdk.adapters.vectordb.vectordb_adapter import VectorDBAdapter
from unstract.sdk.exceptions import VectorDBError


def parse_vector_db_err(e: Exception, vector_db: VectorDBAdapter) -> VectorDBError:
    """Parses the exception from LLM provider.

    Helps parse the LLM error and wraps it with our
    custom exception object to contain a user friendly message.

    Args:
        e (Exception): Error from LLM provider

    Returns:
        LLMError: Unstract's LLMError object
    """
    # Avoid wrapping VectorDBError objects again
    if isinstance(e, VectorDBError):
        return e

    if isinstance(e, QdrantAPIException):
        err = Qdrant.parse_vector_db_err(e)
    else:
        err = VectorDBError(str(e), actual_err=e)

    msg = f"Error from vector DB '{vector_db.get_name()}'."

    # Add a code block only for errors from clients
    if err.actual_err:
        msg += f"\n```\n{str(err)}\n```"
    else:
        msg += str(err)
    err.message = msg
    return err



import re
from json import JSONDecodeError
from pathlib import Path
from typing import Any

from jsonschema import Draft202012Validator, ValidationError, validators

from unstract.sdk.constants import MetadataKey, PropKey
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.tool.mime_types import EXT_MIME_MAP


def extend_with_default(validator_class: Any) -> Any:
    """Extend a JSON schema validator class with a default value functionality.

    Parameters:
    - validator_class (Any): The JSON schema validator class to be extended.

    Returns:
    - Any: The extended JSON schema validator class.

    Example:
    extend_with_default(Draft202012Validator)
    """
    validate_properties = validator_class.VALIDATORS["properties"]

    def set_defaults(
        validator: Any, properties: Any, instance: Any, schema: Any
    ) -> Any:
        for property_, subschema in properties.items():
            if "default" in subschema:
                instance.setdefault(property_, subschema["default"])

        yield from validate_properties(
            validator,
            properties,
            instance,
            schema,
        )

    return validators.extend(
        validator_class,
        {"properties": set_defaults},
    )


# Helps validate a JSON against a schema and applies missing key's defaults too.
DefaultsGeneratingValidator = extend_with_default(Draft202012Validator)


class ToolValidator:
    """Class to validate a tool and its configuration before its executed with
    an input."""

    def __init__(self, tool: BaseTool) -> None:
        self.tool = tool
        props = self.tool.properties
        self.restrictions = props.get(PropKey.RESTRICTIONS)

    def validate_pre_execution(self, settings: dict[str, Any]) -> dict[str, Any]:
        """Performs validation before the tool executes on the input file.

        Args:
            settings (dict[str, Any]): Settings configured for the tool

        Returns:
            dict[str, Any]: Settings JSON for a tool (filled with defaults)
        """
        input_file = Path(self.tool.get_input_file())
        file_exists = self.tool.workflow_filestorage.exists(path=input_file)

        if not file_exists:
            self.tool.stream_error_and_exit(f"Input file not found: {input_file}")
        self._validate_restrictions(input_file)
        self._validate_settings_and_fill_defaults(settings)
        # Call tool's validation hook to execute custom validation
        self.tool.validate(str(input_file), settings)
        return settings

    def _validate_restrictions(self, input_file: Path) -> None:
        """Validates the restrictions mentioned in the tool's PROPERTIES.

        Args:
            input_file (Path): Path object to the input file to be validated
        """
        self._validate_file_size(input_file)
        self._validate_file_type(input_file)

    def _validate_settings_and_fill_defaults(
        self, tool_settings: dict[str, Any]
    ) -> None:
        """Validates and obtains settings for a tool.

        Validation is done against the tool's settings based
        on its declared SPEC. Validator also fills in the missing defaults.

        Args:
            tool_settings (dict[str, Any]): Tool settings to validate
        """
        try:
            spec_schema = self.tool.spec
            DefaultsGeneratingValidator(spec_schema).validate(tool_settings)
        except JSONDecodeError as e:
            self.tool.stream_error_and_exit(f"Settings is not a valid JSON: {str(e)}")
        except ValidationError as e:
            self.tool.stream_error_and_exit(f"Invalid settings: {str(e)}")

    def _validate_file_size(self, input_file: Path) -> None:
        """Validates the input file size against the max allowed size set in
        the tool's PROPERTIES.

        Raises:
            RuntimeError: File size exceeds max allowed size
        """
        max_file_size = self.restrictions.get(PropKey.MAX_FILE_SIZE)
        max_size_in_bytes = self._parse_size_string(max_file_size)

        self.tool.stream_log(
            f"Checking input file size... (max file size: {max_file_size})"
        )
        file_size = self.tool.workflow_filestorage.size(path=input_file)
        self.tool.stream_log(f"Input file size: {self._human_readable_size(file_size)}")

        if file_size > max_size_in_bytes:
            source_name = self.tool.get_exec_metadata.get(MetadataKey.SOURCE_NAME)
            self.tool.stream_error_and_exit(
                f"File {source_name} exceeds the maximum "
                f"allowed size of {max_file_size}"
            )

    def _human_readable_size(self, num: float, suffix: str = "B") -> str:
        """Gets the human readable size for a file,

        Args:
            num (int): Size in bytes to parse
            suffix (str, optional): _description_. Defaults to "B".

        Returns:
            str: Human readable size
        """
        for unit in ("", "K", "M", "G", "T"):
            if abs(num) < 1024.0:
                return f"{num:3.1f}{unit}{suffix}"
            num /= 1024.0
        return f"{num:.1f}{suffix}"

    def _parse_size_string(self, size_string: str) -> int:
        """Parses the size string for validation.

        Args:
            size_string (str): Size string to be parsed

        Raises:
            ValueError: Invalid size format

        Returns:
            int: Size in bytes
        """
        size_match = re.match(PropKey.FILE_SIZE_REGEX, size_string)
        if not size_match:
            self.tool.stream_error_and_exit(
                f"Invalid size string format: {size_string}"
            )

        size, unit = size_match.groups()
        size_in_bytes = int(size)
        if unit.upper() == "KB":
            size_in_bytes *= 1024
        elif unit.upper() == "MB":
            size_in_bytes *= 1024 * 1024
        elif unit.upper() == "GB":
            size_in_bytes *= 1024 * 1024 * 1024
        elif unit.upper() == "TB":
            size_in_bytes *= 1024 * 1024 * 1024 * 1024

        return size_in_bytes

    def _validate_file_type(self, input_file: Path) -> None:
        """Validate the input file type against the allowed types mentioned in
        tool's PROPERTIES.

        Args:
            input_file (Path): Path obj of input file to validate

        Raises:
            RuntimeError: If file type is not supported by the tool
        """
        self.tool.stream_log("Checking input file type...")

        allowed_exts: list[str] = self.restrictions.get(PropKey.ALLOWED_FILE_TYPES)
        allowed_exts = [allowed_type.lower() for allowed_type in allowed_exts]
        if "*" in allowed_exts:
            self.tool.stream_log("Skipping check, tool allows all file types")
            return

        allowed_mimes = []
        for ext in allowed_exts:
            if ext not in EXT_MIME_MAP:
                self.tool.stream_error_and_exit(
                    f"{ext} mentioned in tool PROPERTIES is not supported"
                )
            allowed_mimes.append(EXT_MIME_MAP[ext])
        tool_fs = self.tool.workflow_filestorage
        input_file_mime = tool_fs.mime_type(input_file)
        self.tool.stream_log(f"Input file MIME: {input_file_mime}")
        if input_file_mime not in allowed_mimes:
            self.tool.stream_error_and_exit(
                f"File type of {input_file_mime} is not supported by"
                " the tool, check its PROPERTIES for a list of supported types"
            )



import logging
import os
import pathlib
from typing import Any, Optional

from httpx import ConnectError
from llama_parse import LlamaParse

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.x2text.dto import TextExtractionResult
from unstract.sdk.adapters.x2text.llama_parse.src.constants import LlamaParseConfig
from unstract.sdk.adapters.x2text.x2text_adapter import X2TextAdapter
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class LlamaParseAdapter(X2TextAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("LlamaParse")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "llamaparse|78860239-b3cc-4cc5-b3de-f84315f75d14"

    @staticmethod
    def get_name() -> str:
        return "LlamaParse"

    @staticmethod
    def get_description() -> str:
        return "LlamaParse X2Text"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/llama-parse.png"

     

    def _call_parser(
        self,
        input_file_path: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        parser = LlamaParse(
            api_key=self.config.get(LlamaParseConfig.API_KEY),
            base_url=self.config.get(LlamaParseConfig.BASE_URL),
            result_type=self.config.get(LlamaParseConfig.RESULT_TYPE),
            verbose=self.config.get(LlamaParseConfig.VERBOSE),
            language="en",
            ignore_errors=False,
        )

        try:
            file_extension = pathlib.Path(input_file_path).suffix
            if not file_extension:
                try:
                    input_file_extension = fs.guess_extension(input_file_path)
                    input_file_path_copy = input_file_path
                    input_file_path = ".".join(
                        (input_file_path_copy, input_file_extension)
                    )
                    text_content = fs.read(
                        path=input_file_path_copy, mode="rb", encoding="utf-8"
                    )
                    fs.write(
                        path=input_file_path,
                        data=text_content,
                        mode="wb",
                        encoding="utf-8",
                    )
                except OSError as os_err:
                    logger.error("Exception raised while handling input file.")
                    raise AdapterError(str(os_err))

            file_bytes = fs.read(path=input_file_path, mode="rb")
            documents = parser.load_data(
                file_bytes, extra_info={"file_name": input_file_path}
            )

        except ConnectError as connec_err:
            logger.error(f"Invalid Base URL given. : {connec_err}")
            raise AdapterError(
                "Unable to connect to llama-parse`s service, "
                "please check the Base URL"
            )
        except Exception as exe:
            logger.error(
                "Seems like an invalid API Key or possible internal errors: {exe}"
            )
            raise AdapterError(str(exe))

        response_text = documents[0].text
        return response_text  # type:ignore

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        **kwargs: dict[Any, Any],
    ) -> TextExtractionResult:
        response_text = self._call_parser(input_file_path=input_file_path, fs=fs)
        if output_file_path:
            fs.write(
                path=output_file_path,
                mode="w",
                encoding="utf-8",
                data=response_text,
            )

        return TextExtractionResult(extracted_text=response_text)

    def test_connection(self) -> bool:
        self._call_parser(
            input_file_path=f"{os.path.dirname(__file__)}/static/test_input.doc"
        )
        return True



from .llm_whisperer_v2 import LLMWhispererV2

metadata = {
    "name": LLMWhispererV2.__name__,
    "version": "1.0.0",
    "adapter": LLMWhispererV2,
    "description": "LLMWhispererV2 X2Text adapter",
    "is_active": True,
}



import os
from typing import Any, Optional

from llama_index.core.llms import LLM
from llama_index.llms.bedrock import Bedrock

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.llm.constants import LLMKeys
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter


class Constants:
    MODEL = "model"
    API_KEY = "api_key"
    TIMEOUT = "timeout"
    MAX_RETRIES = "max_retries"
    SECRET_ACCESS_KEY = "aws_secret_access_key"
    ACCESS_KEY_ID = "aws_access_key_id"
    REGION_NAME = "region_name"
    CONTEXT_SIZE = "context_size"
    MAX_TOKENS = "max_tokens"
    DEFAULT_MAX_TOKENS = 512  # Default at llama-index


class BedrockLLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("Bedrock")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "bedrock|8d18571f-5e96-4505-bd28-ad0379c64064"

    @staticmethod
    def get_name() -> str:
        return "Bedrock"

    @staticmethod
    def get_description() -> str:
        return "Bedrock LLM"

    @staticmethod
    def get_provider() -> str:
        return "bedrock"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/Bedrock.png"  

    def get_llm_instance(self) -> LLM:
        try:
            context_size: Optional[int] = (
                int(self.config.get(Constants.CONTEXT_SIZE, 0))
                if self.config.get(Constants.CONTEXT_SIZE)
                else None
            )
            max_tokens = int(
                self.config.get(Constants.MAX_TOKENS, Constants.DEFAULT_MAX_TOKENS)
            )
            llm: LLM = Bedrock(
                model=self.config.get(Constants.MODEL),
                aws_access_key_id=self.config.get(Constants.ACCESS_KEY_ID),
                aws_secret_access_key=self.config.get(Constants.SECRET_ACCESS_KEY),
                region_name=self.config.get(Constants.REGION_NAME),
                timeout=float(
                    self.config.get(Constants.TIMEOUT, LLMKeys.DEFAULT_TIMEOUT)
                ),
                max_retries=int(
                    self.config.get(Constants.MAX_RETRIES, LLMKeys.DEFAULT_MAX_RETRIES)
                ),
                temperature=0,
                context_size=context_size,
                max_tokens=max_tokens,
            )
            return llm
        except Exception as e:
            raise AdapterError(str(e))


import json
import logging
import os
import unittest
from typing import Any

from dotenv import load_dotenv
from parameterized import parameterized

from unstract.sdk.ocr import OCR
from unstract.sdk.tool.base import BaseTool

load_dotenv()

logger = logging.getLogger(__name__)


def get_test_values(env_key: str) -> list[str]:
    values = json.loads(os.environ.get(env_key))
    return values


def get_env_value(env_key: str) -> str:
    value = os.environ.get(env_key)
    return value


class ToolOCRTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            pass

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool()

    @parameterized.expand(get_test_values("OCR_TEST_VALUES"))
    def test_get_ocr(self, adapter_instance_id):
        tool_ocr = OCR(tool=self.tool)
        ocr = tool_ocr.get_ocr(adapter_instance_id)
        result = ocr.test_connection()
        self.assertTrue(result)
        input_file = get_env_value("INPUT_FILE_PATH")
        output_file = get_env_value("OUTPUT_FILE_PATH")
        if os.path.isfile(output_file):
            os.remove(output_file)
        output = ocr.process(input_file, output_file)
        file_size = os.path.getsize(output_file)
        self.assertGreater(file_size, 0)
        if os.path.isfile(output_file):
            os.remove(output_file)
        with open(output_file, "w", encoding="utf-8") as f:
            f.write(output)
            f.close()
        file_size = os.path.getsize(output_file)
        self.assertGreater(file_size, 0)


if __name__ == "__main__":
    unittest.main()



from unstract.sdk.tool.base import BaseTool
from unstract.sdk.tool.executor import ToolExecutor
from unstract.sdk.tool.parser import ToolArgsParser


class ToolEntrypoint:
    """Class that contains methods for the entrypoint for a tool."""

    @staticmethod
    def launch(tool: BaseTool, args: list[str]) -> None:
        """Entrypoint function for a tool.

        It parses the arguments passed to a tool and executes
        the intended command.

        Args:
            tool (AbstractTool): Tool to execute
            args (List[str]): Arguments passed to a tool
        """
        parsed_args = ToolArgsParser.parse_args(args)
        executor = ToolExecutor(tool=tool)
        executor.execute(parsed_args)



from abc import ABC, abstractmethod
from typing import Any


class AdapterRegistry(ABC):
    def __init__(self, name: str):
        self.name = name

    @staticmethod
    @abstractmethod
    def register_adapters(adapters: dict[str, Any]) -> None:
        pass



from .unstructured_enterprise import UnstructuredEnterprise

metadata = {
    "name": UnstructuredEnterprise.__name__,
    "version": "1.0.0",
    "adapter": UnstructuredEnterprise,
    "description": "UnstructuredIO X2Text adapter",
    "is_active": True,
}



from typing import Any, Optional

from deprecated import deprecated
from llama_index.core.base.embeddings.base import Embedding
from llama_index.core.callbacks import CallbackManager as LlamaIndexCallbackManager
from llama_index.core.embeddings import BaseEmbedding

from unstract.sdk.adapter import ToolAdapter
from unstract.sdk.adapters.constants import Common
from unstract.sdk.adapters.embedding import adapters
from unstract.sdk.constants import LogLevel, ToolEnv
from unstract.sdk.exceptions import EmbeddingError, SdkError
from unstract.sdk.helper import SdkHelper
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.utils.callback_manager import CallbackManager


class Embedding:
    _TEST_SNIPPET = "Hello, I am Unstract"
    MAX_TOKENS = 1024 * 16
    embedding_adapters = adapters

    def __init__(
        self,
        tool: BaseTool,
        adapter_instance_id: Optional[str] = None,
        usage_kwargs: dict[Any, Any] = {},
    ):
        self._tool = tool
        self._adapter_instance_id = adapter_instance_id
        self._embedding_instance: BaseEmbedding = None
        self._length: int = None
        self._usage_kwargs = usage_kwargs
        self._initialise()

    def _initialise(self):
        if self._adapter_instance_id:
            self._embedding_instance = self._get_embedding()
            self._length: int = self._get_embedding_length()
            self._usage_kwargs["adapter_instance_id"] = self._adapter_instance_id

            if not SdkHelper.is_public_adapter(adapter_id=self._adapter_instance_id):
                platform_api_key = self._tool.get_env_or_die(ToolEnv.PLATFORM_API_KEY)
                CallbackManager.set_callback(
                    platform_api_key=platform_api_key,
                    model=self._embedding_instance,
                    kwargs=self._usage_kwargs,
                )

    def _get_embedding(self) -> BaseEmbedding:
        """Gets an instance of LlamaIndex's embedding object.

        Args:
            adapter_instance_id (str): UUID of the embedding adapter

        Returns:
            BaseEmbedding: Embedding instance
        """
        try:
            if not self._adapter_instance_id:
                raise EmbeddingError(
                    "Adapter instance ID not set. " "Initialisation failed"
                )

            embedding_config_data = ToolAdapter.get_adapter_config(
                self._tool, self._adapter_instance_id
            )
            embedding_adapter_id = embedding_config_data.get(Common.ADAPTER_ID)
            if embedding_adapter_id not in self.embedding_adapters:
                raise SdkError(
                    f"Embedding adapter not supported : " f"{embedding_adapter_id}"
                )

            embedding_adapter = self.embedding_adapters[embedding_adapter_id][
                Common.METADATA
            ][Common.ADAPTER]
            embedding_metadata = embedding_config_data.get(Common.ADAPTER_METADATA)
            embedding_adapter_class = embedding_adapter(embedding_metadata)
            self._usage_kwargs["provider"] = embedding_adapter_class.get_provider()
            return embedding_adapter_class.get_embedding_instance()
        except Exception as e:
            self._tool.stream_log(
                log=f"Error getting embedding: {e}", level=LogLevel.ERROR
            )
            raise EmbeddingError(f"Error getting embedding instance: {e}") from e

    def get_query_embedding(self, query: str) -> Embedding:
        return self._embedding_instance.get_query_embedding(query)

    def _get_embedding_length(self) -> int:
        embedding_list = self._embedding_instance._get_text_embedding(
            self._TEST_SNIPPET
        )
        embedding_dimension = len(embedding_list)
        return embedding_dimension

    def get_class_name(self) -> str:
        """Gets the class name of the Llama Index Embedding.

        Args:
            NA

            Returns:
                Class name
        """
        return self._embedding_instance.class_name()

    def get_callback_manager(self) -> LlamaIndexCallbackManager:
        """Gets the llama-index callback manager set on the model.

        Args:
            NA

            Returns:
                llama-index callback manager
        """
        return self._embedding_instance.callback_manager

    @deprecated("Use Embedding instead of ToolEmbedding")
    def get_embedding_length(self, embedding: BaseEmbedding) -> int:
        return self._get_embedding_length()

    @deprecated("Use Embedding instead of ToolEmbedding")
    def get_embedding(self, adapter_instance_id: str) -> BaseEmbedding:
        if not self._embedding_instance:
            self._adapter_instance_id = adapter_instance_id
            self._initialise()
        return self._embedding_instance


# Legacy
ToolEmbedding = Embedding



from enum import Enum


class FileOperationParams:
    READ_ENTIRE_LENGTH = -1
    EXTENSION_DEFAULT_READ_LENGTH = 100
    DEFAULT_ENCODING = "utf-8"


class FileSeekPosition:
    START = 0
    CURRENT = 1
    END = 2


class StorageType(Enum):
    PERMANENT = "permanent"
    SHARED_TEMPORARY = "shared_temporary"


class CredentialKeyword:
    PROVIDER = "provider"
    CREDENTIALS = "credentials"



from .mistral import MistralLLM

metadata = {
    "name": MistralLLM.__name__,
    "version": "1.0.0",
    "adapter": MistralLLM,
    "description": "Mistral LLM adapter",
    "is_active": True,
}

__all__ = ["MistralLLM"]



import logging
import os
from typing import Any, Optional

from llama_index.core.schema import BaseNode
from llama_index.core.vector_stores.types import BasePydanticVectorStore
from llama_index.vector_stores.pinecone import PineconeVectorStore
from pinecone import NotFoundException
from pinecone import Pinecone as LLamaIndexPinecone
from pinecone import PodSpec, ServerlessSpec

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.adapters.vectordb.helper import VectorDBHelper
from unstract.sdk.adapters.vectordb.vectordb_adapter import VectorDBAdapter

logger = logging.getLogger(__name__)


class Constants:
    API_KEY = "api_key"
    ENVIRONMENT = "environment"
    NAMESPACE = "namespace"
    DIMENSION = 1536
    METRIC = "euclidean"
    SPECIFICATION = "spec"
    SPEC_POD = "pod"
    SPEC_SERVERLESS = "serverless"
    CLOUD = "cloud"
    REGION = "region"
    DEFAULT_SPEC_COUNT_VALUE = 1
    DEFAULT_POD_TYPE = "p1.x1"


class Pinecone(VectorDBAdapter):
    def __init__(self, settings: dict[str, Any]):
        self._config = settings
        self._client: Optional[LLamaIndexPinecone] = None
        self._collection_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        self._vector_db_instance = self._get_vector_db_instance()
        super().__init__("Pinecone", self._vector_db_instance)

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "pinecone|83881133-485d-4ecc-b1f7-0009f96dc74a"

    @staticmethod
    def get_name() -> str:
        return "Pinecone"

    @staticmethod
    def get_description() -> str:
        return "Pinecone VectorDB"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/pinecone.png"

     

    def get_vector_db_instance(self) -> BasePydanticVectorStore:
        return self._vector_db_instance

    def _get_vector_db_instance(self) -> BasePydanticVectorStore:
        try:
            self._client = LLamaIndexPinecone(
                api_key=str(self._config.get(Constants.API_KEY))
            )
            dimension = self._config.get(
                VectorDbConstants.EMBEDDING_DIMENSION,
                VectorDbConstants.DEFAULT_EMBEDDING_SIZE,
            )
            collection_name = VectorDBHelper.get_collection_name(
                self._config.get(VectorDbConstants.VECTOR_DB_NAME),
                dimension,
            )
            self._collection_name = collection_name.replace("_", "-").lower()

            specification = self._config.get(Constants.SPECIFICATION)
            if specification == Constants.SPEC_POD:
                environment = self._config.get(Constants.ENVIRONMENT)
                spec = PodSpec(
                    environment=environment,
                    replicas=Constants.DEFAULT_SPEC_COUNT_VALUE,
                    shards=Constants.DEFAULT_SPEC_COUNT_VALUE,
                    pods=Constants.DEFAULT_SPEC_COUNT_VALUE,
                    pod_type=Constants.DEFAULT_POD_TYPE,
                )
            elif specification == Constants.SPEC_SERVERLESS:
                cloud = self._config.get(Constants.CLOUD)
                region = self._config.get(Constants.REGION)
                spec = ServerlessSpec(cloud=cloud, region=region)
            logger.info(f"Setting up Pinecone spec for {spec}")
            try:
                self._client.describe_index(name=self._collection_name)
            except NotFoundException:
                logger.info(
                    f"Index:{self._collection_name} does not exist. Creating it."
                )
                self._client.create_index(
                    name=self._collection_name,
                    dimension=dimension,
                    metric=Constants.METRIC,
                    spec=spec,
                )
            self.vector_db: BasePydanticVectorStore = PineconeVectorStore(
                index_name=self._collection_name,
                api_key=str(self._config.get(Constants.API_KEY)),
                environment=str(self._config.get(Constants.ENVIRONMENT)),
            )
            return self.vector_db
        except Exception as e:
            raise AdapterError(str(e))

    def test_connection(self) -> bool:
        vector_db = self.get_vector_db_instance()
        test_result: bool = VectorDBHelper.test_vector_db_instance(
            vector_store=vector_db
        )
        # Delete the collection that was created for testing
        if self._client:
            self._client.delete_index(self._collection_name)
        return test_result

    def close(self, **kwargs: Any) -> None:
        # Close connection is not defined for this client
        pass

    def delete(self, ref_doc_id: str, **delete_kwargs: dict[Any, Any]) -> None:
        specification = self._config.get(Constants.SPECIFICATION)
        if specification == Constants.SPEC_SERVERLESS:
            # To delete all records representing chunks of a single document,
            # first list the record IDs based on their common ID prefix,
            # and then delete the records by ID:
            try:
                index = self._client.Index(self._collection_name)  # type: ignore
                # Get all record having the ref_doc_id and delete them
                for ids in index.list(prefix=ref_doc_id):
                    logger.info(ids)
                    index.delete(ids=ids)
            except Exception as e:
                raise AdapterError(str(e))
        elif specification == Constants.SPEC_POD:
            if self.vector_db.environment == "gcp-starter":  # type: ignore
                raise AdapterError(
                    "Re-indexing is not supported on Starter indexes. "
                    "Use Serverless or paid plan for Pod spec"
                )
            else:
                super().delete(ref_doc_id=ref_doc_id, **delete_kwargs)

    def add(
        self,
        ref_doc_id: str,
        nodes: list[BaseNode],
    ) -> list[str]:
        for i, node in enumerate(nodes):
            node_id = ref_doc_id + "-" + node.node_id
            nodes[i].id_ = node_id
        return self.vector_db.add(nodes=nodes)



from .anyscale import AnyScaleLLM

metadata = {
    "name": AnyScaleLLM.__name__,
    "version": "1.0.0",
    "adapter": AnyScaleLLM,
    "description": "AnyScale LLM adapter",
    "is_active": True,
}

__all__ = ["AnyScaleLLM"]



import time
from typing import Any

from llama_index.core.base.llms.types import (
    CompletionResponse,
    CompletionResponseGen,
    LLMMetadata,
)
from llama_index.core.llms.custom import CustomLLM


class NoOpCustomLLM(CustomLLM):
    wait_time: float

    def __init__(
        self,
        wait_time: float,
    ) -> None:
        wait_time = wait_time
        super().__init__(wait_time=wait_time)

    @classmethod
    def class_name(cls) -> str:
        return "NoOpLLM"

    def _generate_text(self) -> str:
        # Returns a JSON here to support for all enforce types.
        return '{ "response":"This is a sample response from a NoOp LLM Adapter."}'

    def complete(
        self, prompt: str, formatted: bool = False, **kwargs: Any
    ) -> CompletionResponse:
        time.sleep(self.wait_time)
        response_text = self._generate_text()

        return CompletionResponse(
            text=response_text,
        )

    def stream_complete(
        self, prompt: str, formatted: bool = False, **kwargs: Any
    ) -> CompletionResponseGen:

        def gen_response() -> CompletionResponseGen:
            response_text = self._generate_text()
            yield CompletionResponse(
                text=response_text,
                delta=response_text,
            )

        time.sleep(self.wait_time)

        return gen_response()

    @property
    def metadata(self) -> LLMMetadata:
        """Method to fetch LLM metadata. Overriden to extent Base class.

        Returns:
            LLMMetadata
        """
        return LLMMetadata(num_output=-1)



import os
from typing import Any, Optional

from llama_index.core.vector_stores.types import VectorStore
from llama_index.vector_stores.milvus import MilvusVectorStore
from pymilvus import MilvusClient

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.adapters.vectordb.helper import VectorDBHelper
from unstract.sdk.adapters.vectordb.vectordb_adapter import VectorDBAdapter


class Constants:
    URI = "uri"
    TOKEN = "token"
    DIM_VALUE = 1536


class Milvus(VectorDBAdapter):
    def __init__(self, settings: dict[str, Any]):
        self._config = settings
        self._client: Optional[MilvusClient] = None
        self._collection_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        self._vector_db_instance = self._get_vector_db_instance()
        super().__init__("Milvus", self._vector_db_instance)

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "milvus|3f42f6f9-4b8e-4546-95f3-22ecc9aca442"

    @staticmethod
    def get_name() -> str:
        return "Milvus"

    @staticmethod
    def get_description() -> str:
        return "Milvus VectorDB"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/Milvus.png"

     

    def get_vector_db_instance(self) -> VectorStore:
        return self._vector_db_instance

    def _get_vector_db_instance(self) -> VectorStore:
        try:
            dimension = self._config.get(
                VectorDbConstants.EMBEDDING_DIMENSION,
                VectorDbConstants.DEFAULT_EMBEDDING_SIZE,
            )
            self._collection_name = VectorDBHelper.get_collection_name(
                self._config.get(VectorDbConstants.VECTOR_DB_NAME),
                dimension,
            )
            vector_db: VectorStore = MilvusVectorStore(
                uri=self._config.get(Constants.URI, ""),
                collection_name=self._collection_name,
                token=self._config.get(Constants.TOKEN, ""),
                dim=dimension,
            )
            if vector_db is not None:
                self._client = vector_db.client
            return vector_db
        except Exception as e:
            raise AdapterError(str(e))

    def test_connection(self) -> bool:
        vector_db = self.get_vector_db_instance()
        test_result: bool = VectorDBHelper.test_vector_db_instance(
            vector_store=vector_db
        )
        # Delete the collection that was created for testing
        if self._client is not None:
            self._client.drop_collection(self._collection_name)
        return test_result

    def close(self, **kwargs: Any) -> None:
        if self._client:
            self._client.close()



import os
from typing import Any, Optional

from google.api_core.exceptions import GoogleAPICallError
from llama_index.core.llms import LLM
from llama_index.llms.palm import PaLM

from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter
from unstract.sdk.exceptions import LLMError


class Constants:
    MODEL = "model_name"
    API_KEY = "api_key"
    NUM_OUTPUT = "num_output"
    API_TYPE = "palm"
    DEFAULT_MAX_TOKENS = 1024


class PaLMLLM(LLMAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("PaLM")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "palm|af7c8ee7-3d01-47c5-9b81-5ffd7546014b"

    @staticmethod
    def get_name() -> str:
        return "Palm"

    @staticmethod
    def get_description() -> str:
        return "Palm LLM"

    @staticmethod
    def get_provider() -> str:
        return "palm"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/PaLM.png"

     

    def get_llm_instance(self) -> LLM:
        try:
            num_output: Optional[int] = (
                int(self.config.get(Constants.NUM_OUTPUT, Constants.DEFAULT_MAX_TOKENS))
                if self.config.get(Constants.NUM_OUTPUT) is not None
                else None
            )
            llm: LLM = PaLM(
                model=str(self.config.get(Constants.MODEL)),
                api_key=str(self.config.get(Constants.API_KEY)),
                num_output=num_output,
                api_type=Constants.API_TYPE,
                temperature=0,
            )
            return llm
        except Exception as e:
            # To avoid circular import errors
            from unstract.sdk.adapters.llm.exceptions import parse_llm_err

            raise parse_llm_err(e, llm_adapter=self)

    @staticmethod
    def parse_llm_err(e: GoogleAPICallError) -> LLMError:
        """Parse the error from PaLM.

        Helps parse errors from PaLM and wraps with custom exception.

        Args:
            e (OpenAIAPIError): Exception from PaLM

        Returns:
            LLMError: Error to be sent to the user
        """
        return LLMError(f"{e.message}", actual_err=e)



class LlamaParseConfig:
    """Dictionary keys used to process LlamaParse."""

    API_KEY = "api_key"
    BASE_URL = "base_url"
    RESULT_TYPE = "result_type"
    NUM_WORKERS = "num_workers"
    VERBOSE = "verbose"
    LANGUAGE = "language"



from abc import ABC
from typing import Any, Union

from llama_index.core.schema import BaseNode
from llama_index.core.vector_stores import SimpleVectorStore
from llama_index.core.vector_stores.types import BasePydanticVectorStore, VectorStore

from unstract.sdk.adapters.base import Adapter
from unstract.sdk.adapters.enums import AdapterTypes
from unstract.sdk.exceptions import VectorDBError


class VectorDBAdapter(Adapter, ABC):
    def __init__(
        self,
        name: str,
        vector_db_instance: Union[VectorStore, BasePydanticVectorStore],
    ):
        super().__init__(name)
        self.name = name
        self._vector_db_instance: Union[
            VectorStore, BasePydanticVectorStore
        ] = vector_db_instance

    @staticmethod
    def get_id() -> str:
        return ""

    @staticmethod
    def get_name() -> str:
        return ""

    @staticmethod
    def get_description() -> str:
        return ""

    @staticmethod
    def get_icon() -> str:
        return ""

    @staticmethod
    def get_adapter_type() -> AdapterTypes:
        return AdapterTypes.VECTOR_DB

    @staticmethod
    def parse_vector_db_err(e: Exception) -> VectorDBError:
        """Parse the error from a vector DB.

        Helps parse errors from a vector DB and wraps with custom exception.

        Args:
            e (Exception): Exception from vector DB

        Returns:
            VectorDBError: Error to be sent to the user
        """
        return VectorDBError(str(e), actual_err=e)

    def get_vector_db_instance(
        self, vector_db_config: dict[str, Any]
    ) -> Union[BasePydanticVectorStore, VectorStore]:
        """Instantiate the llama index VectorStore / BasePydanticVectorStore
        class.

        Returns:
            BasePydanticVectorStore / VectorStore:
                            llama index implementation of the vector store
            Raises exceptions for any error
        """
        return SimpleVectorStore()

    def close(self, **kwargs: Any) -> None:
        """Closes the client connection.

        Returns:
            None
        """
        # Overriding implementations will have the corresponding
        # library methods invoked
        pass

    def delete(self, ref_doc_id: str, **delete_kwargs: Any) -> None:
        """Delete the specified docs.

        Returns:
            None
        """
        # Overriding implementations will have the corresponding
        # library methods invoked
        self._vector_db_instance.delete(
            ref_doc_id=ref_doc_id, delete_kwargs=delete_kwargs
        )

    def add(self, ref_doc_id: str, nodes: list[BaseNode]) -> list[str]:
        return self._vector_db_instance.add(nodes=nodes)



from .no_op_embedding import NoOpEmbedding

metadata = {
    "name": NoOpEmbedding.__name__,
    "version": "1.0.0",
    "adapter": NoOpEmbedding,
    "description": "NoOp embedding adapter",
    "is_active": True,
}



import os
from typing import Any, Optional

from llama_index.core.embeddings import BaseEmbedding
from llama_index.embeddings.huggingface import HuggingFaceEmbedding

from unstract.sdk.adapters.embedding.embedding_adapter import EmbeddingAdapter
from unstract.sdk.adapters.embedding.helper import EmbeddingHelper
from unstract.sdk.adapters.exceptions import AdapterError


class Constants:
    ADAPTER_NAME = "adapter_name"
    MODEL = "model_name"
    TOKENIZER_NAME = "tokenizer_name"
    MAX_LENGTH = "max_length"
    NORMALIZE = "normalize"


class HuggingFace(EmbeddingAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("HuggingFace")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "huggingface|90ec9ec2-1768-4d69-8fb1-c88b95de5e5a"

    @staticmethod
    def get_name() -> str:
        return "HuggingFace"

    @staticmethod
    def get_description() -> str:
        return "HuggingFace Embedding"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/huggingface.png"

    def get_embedding_instance(self) -> BaseEmbedding:
        try:
            embedding_batch_size = EmbeddingHelper.get_embedding_batch_size(
                config=self.config
            )
            max_length: Optional[int] = (
                int(self.config.get(Constants.MAX_LENGTH, 0))
                if self.config.get(Constants.MAX_LENGTH)
                else None
            )
            embedding: BaseEmbedding = HuggingFaceEmbedding(
                model_name=str(self.config.get(Constants.MODEL)),
                tokenizer_name=str(self.config.get(Constants.TOKENIZER_NAME)),
                normalize=bool(self.config.get(Constants.NORMALIZE)),
                embed_batch_size=embedding_batch_size,
                max_length=max_length,
            )

            return embedding
        except Exception as e:
            raise AdapterError(str(e))

     



from .azure_open_ai import AzureOpenAI

metadata = {
    "name": AzureOpenAI.__name__,
    "version": "1.0.0",
    "adapter": AzureOpenAI,
    "description": "AzureOpenAI embedding adapter",
    "is_active": True,
}



import logging
from typing import Any, Optional, Union

import filetype
import magic

from unstract.sdk.exceptions import FileOperationError, FileStorageError
from unstract.sdk.file_storage.constants import FileOperationParams
from unstract.sdk.file_storage.impl import FileStorage
from unstract.sdk.file_storage.provider import FileStorageProvider

logger = logging.getLogger(__name__)


class PermanentFileStorage(FileStorage):
    SUPPORTED_FILE_STORAGE_TYPES = [
        FileStorageProvider.GCS.value,
        FileStorageProvider.S3.value,
        FileStorageProvider.MINIO.value,
        FileStorageProvider.LOCAL.value,
        FileStorageProvider.AZURE.value,
    ]

    def __init__(
        self,
        provider: FileStorageProvider,
        **storage_config: dict[str, Any],
    ):
        if provider.value not in self.SUPPORTED_FILE_STORAGE_TYPES:
            raise FileStorageError(
                f"File storage provider `{provider.value}` is not "
                f"supported in Permanent mode. "
                f"Supported providers: {self.SUPPORTED_FILE_STORAGE_TYPES}"
            )
        if (
            provider == FileStorageProvider.GCS
            or provider == FileStorageProvider.LOCAL
            or provider == FileStorageProvider.MINIO
            or provider == FileStorageProvider.S3
            or provider == FileStorageProvider.AZURE
        ):
            super().__init__(provider, **storage_config)
        else:
            raise NotImplementedError(f"Provider {provider.value} is not implemented")

    def _copy_on_read(self, path: str, legacy_storage_path: str):
        """Copies the file to the remote storage lazily if not present already.
        Checks if the file is present in the Local File system. If yes, copies
        the file to the mentioned path using the remote file system. This is a
        silent copy done on need basis.

        Args:
            path (str): Path to the file
            legacy_storage_path (str): Legacy path to the same file

        Returns:
            NA
        """
        # If path does not exist on remote storage
        if not self.exists(path):
            local_file_storage = FileStorage(provider=FileStorageProvider.LOCAL)
            local_file_path = legacy_storage_path
            # If file exists on local storage, then migrate the file
            # to remote storage
            if local_file_storage.exists(local_file_path):
                self.upload(local_file_path, path)
                logger.info(
                    f"Uploading {local_file_path} from "
                    f"{local_file_storage.provider} to remote "
                    f"storage {self.provider} in the path {path}"
                )

    def read(
        self,
        path: str,
        mode: str,
        encoding: str = FileOperationParams.DEFAULT_ENCODING,
        seek_position: int = 0,
        length: int = FileOperationParams.READ_ENTIRE_LENGTH,
        legacy_storage_path: Optional[str] = None,
    ) -> Union[bytes, str]:
        """Read the file pointed to by the file_handle.

        Args:
            path (str): Path to the file to be opened
            mode (str): Mode in whicg the file is to be opened. Usual options
                        include r, rb, w and wb
            encoding (str): Encoding type like utf-8 or utf-16
            seek_position (int): Position to start reading from
            length (int): Number of bytes to be read. Default is full
            file content.
            legacy_storage_path (str):  Legacy path to the same file

        Returns:
            Union[bytes, str] - File contents in bytes/string based on the opened mode
        """
        try:
            # Lazy copy to the destination/remote file system
            if legacy_storage_path:
                self._copy_on_read(path, legacy_storage_path)
            return super().read(path, mode, encoding, seek_position, length)
        except Exception as e:
            if isinstance(e, FileNotFoundError) or isinstance(e, FileOperationError):
                raise e
            raise FileOperationError(str(e)) from e

    def mime_type(
        self,
        path: str,
        read_length: int = FileOperationParams.READ_ENTIRE_LENGTH,
        legacy_storage_path: Optional[str] = None,
    ) -> str:
        """Gets the file MIME type for an input file. Uses libmagic to perform
        the same.

        Args:
            path (str): Path of the input file
            read_length (int): Length(bytes) to be read from the file for in
            order to identify the mime type. Defaults to read the entire length.
            legacy_storage_path (str):  Legacy path to the same file

        Returns:
            str: MIME type of the file
        """
        sample_contents = self.read(
            path=path,
            mode="rb",
            length=read_length,
            legacy_storage_path=legacy_storage_path,
        )
        mime_type = magic.from_buffer(sample_contents, mime=True)
        return mime_type

    def guess_extension(
        self, path: str, legacy_storage_path: Optional[str] = None
    ) -> str:
        """Returns the extension of the file passed.

        Args:
            path (str): String holding the path
            legacy_storage_path (str):  Legacy path to the same file

        Returns:
            str: File extension
        """
        file_extension = ""
        sample_contents = self.read(
            path=path,
            mode="rb",
            length=FileOperationParams.EXTENSION_DEFAULT_READ_LENGTH,
            legacy_storage_path=legacy_storage_path,
        )
        if sample_contents:
            file_type = filetype.guess(sample_contents)
            file_extension = file_type.EXTENSION
        return file_extension



import unittest
from io import StringIO
from typing import Any
from unittest.mock import patch

from unstract.sdk.constants import LogLevel
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.tool.entrypoint import ToolEntrypoint


class UnstractSDKToolsEntrypointTest(unittest.TestCase):
    INFO_MESSAGE = "Running a mock tool"
    DEBUG_MESSAGE = "Example DEBUG message"

    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            self.stream_log(UnstractSDKToolsEntrypointTest.INFO_MESSAGE)
            self.stream_log(
                UnstractSDKToolsEntrypointTest.DEBUG_MESSAGE, LogLevel.DEBUG
            )

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool(log_level=LogLevel.DEBUG)

    def _launch_tool(self, args: list[str]) -> str:
        captured_output = StringIO()
        with patch("sys.stdout", new=captured_output):
            ToolEntrypoint.launch(tool=self.tool, args=args)
        return captured_output.getvalue()

    def test_spec(self):
        args = [
            "--command",
            "SPEC",
            "--workflow-id",
            "00000000-0000-0000-0000-000000000000",
            "--log-level",
            "INFO",
        ]
        captured_output_str = self._launch_tool(args=args)
        self.assertIn("SPEC", captured_output_str)

    def test_properties(self):
        args = [
            "--command",
            "PROPERTIES",
            "--workflow-id",
            "00000000-0000-0000-0000-000000000000",
            "--log-level",
            "INFO",
        ]
        captured_output_str = self._launch_tool(args=args)
        self.assertIn("PROPERTIES", captured_output_str)

    def test_variables(self):
        args = [
            "--command",
            "VARIABLES",
            "--workflow-id",
            "00000000-0000-0000-0000-000000000000",
            "--log-level",
            "INFO",
        ]
        captured_output_str = self._launch_tool(args=args)
        self.assertIn("VARIABLES", captured_output_str)

    def test_run(self):
        args = [
            "--command",
            "RUN",
            "--params",
            "{}",
            "--settings",
            "{}",
            "--workflow-id",
            "00000000-0000-0000-0000-000000000000",
            "--log-level",
            "INFO",
        ]
        captured_output_str = self._launch_tool(args=args)
        self.assertIn(self.INFO_MESSAGE, captured_output_str)

    def test_run_debug(self):
        args = [
            "--command",
            "RUN",
            "--params",
            "{}",
            "--settings",
            "{}",
            "--workflow-id",
            "00000000-0000-0000-0000-000000000000",
            "--log-level",
            "DEBUG",
        ]
        captured_output_str = self._launch_tool(args=args)
        self.assertIn(self.DEBUG_MESSAGE, captured_output_str)


if __name__ == "__main__":
    unittest.main()



import logging

from unstract.sdk.file_storage.impl import FileStorage

logger = logging.getLogger(__name__)


class FileStorageUtils:
    @staticmethod
    def copy_file_to_destination(
        source_storage: FileStorage,
        destination_storage: FileStorage,
        source_path: str,
        destination_paths: list[str],
        chunk_size: int = 4096,
    ) -> None:
        """Copy a file from a source storage to one or more paths in a
        destination storage.

        This function reads the source file in chunks and writes each chunk to
        the specified destination paths. The function will continue until the
        entire source file is copied.

        Args:
            source_storage (FileStorage): The storage object from which
                the file is read.
            destination_storage (FileStorage): The storage object to which
                the file is written.
            source_path (str): The path of the file in the source storage.
            destination_paths (list[str]): A list of paths where the file will be
                copied in the destination storage.
            chunk_size (int, optional): The number of bytes to read per chunk.
                Default is 4096 bytes.
        """
        seek_position = 0  # Start from the beginning
        end_of_file = False

        # Loop to read and write in chunks until the end of the file
        while not end_of_file:
            # Read a chunk from the source file
            chunk = source_storage.read(
                path=source_path,
                mode="rb",
                seek_position=seek_position,
                length=chunk_size,
            )
            # Check if the end of the file has been reached
            if not chunk:
                end_of_file = True
            else:
                # Write the chunk to each destination path
                for destination_file in destination_paths:
                    destination_storage.write(
                        path=destination_file,
                        mode="ab",
                        seek_position=seek_position,
                        data=chunk,
                    )

                # Update the seek position
                seek_position += len(chunk)



from .qdrant_fast_embed import QdrantFastEmbedM

metadata = {
    "name": QdrantFastEmbedM.__name__,
    "version": "1.0.0",
    "adapter": QdrantFastEmbedM,
    "description": "QdrantFastEmbed embedding adapter",
    "is_active": False,
}



import datetime
import json
import logging
import os
from typing import Any

from deprecated import deprecated

from unstract.sdk.constants import Command, LogLevel, LogStage, ToolEnv
from unstract.sdk.utils import ToolUtils
from unstract.sdk.utils.common_utils import UNSTRACT_TO_PY_LOG_LEVEL


class StreamMixin:
    """Helper class for streaming Unstract tool commands.

    A utility class to make writing Unstract tools easier. It provides
    methods to stream the JSON schema, properties, icon, log messages,
    cost, single step messages, and results using the Unstract protocol
    to stdout.
    """

    def __init__(self, log_level: LogLevel = LogLevel.INFO, **kwargs) -> None:
        """
        Args:
            log_level (LogLevel): The log level for filtering of log messages.
            The default is INFO.
                Allowed values are DEBUG, INFO, WARN, ERROR, and FATAL.

        """
        self.log_level = log_level
        self._exec_by_tool = ToolUtils.str_to_bool(
            os.environ.get(ToolEnv.EXECUTION_BY_TOOL, "False")
        )
        if self.is_exec_by_tool:
            self._configure_logger()
        super().__init__(**kwargs)

    @property
    def is_exec_by_tool(self):
        """Flag to determine if SDK library is used in a tool's context.

        Returns:
            bool: True if SDK is used by a tool else False
        """
        return self._exec_by_tool

    def _configure_logger(self) -> None:
        """Helps configure the logger for the tool run."""
        rootlogger = logging.getLogger("")
        # Avoids adding multiple handlers
        if rootlogger.hasHandlers():
            return
        handler = logging.StreamHandler()
        handler.setLevel(level=UNSTRACT_TO_PY_LOG_LEVEL[self.log_level])
        handler.setFormatter(
            logging.Formatter(
                "[%(asctime)s] %(levelname)s in %(module)s: %(message)s",
            )
        )
        rootlogger.addHandler(handler)
        rootlogger.setLevel(level=UNSTRACT_TO_PY_LOG_LEVEL[self.log_level])

    def stream_log(
        self,
        log: str,
        level: LogLevel = LogLevel.INFO,
        stage: str = LogStage.TOOL_RUN,
        **kwargs: Any,
    ) -> None:
        """Streams a log message using the Unstract protocol LOG to stdout.

        Args:
            log (str): The log message.
            level (LogLevel): The log level. The default is INFO.
                Allowed values are DEBUG, INFO, WARN, ERROR, and FATAL.
            stage (str): LogStage from constant default Tool_RUN
        Returns:
            None
        """
        levels = [
            LogLevel.DEBUG,
            LogLevel.INFO,
            LogLevel.WARN,
            LogLevel.ERROR,
            LogLevel.FATAL,
        ]
        if levels.index(level) < levels.index(self.log_level):
            return

        record = {
            "type": "LOG",
            "stage": stage,
            "level": level.value,
            "log": log,
            "emitted_at": datetime.datetime.now().isoformat(),
            **kwargs,
        }
        print(json.dumps(record))

    def stream_error_and_exit(self, message: str) -> None:
        """Stream error log and exit.

        Args:
            message (str): Error message
        """
        self.stream_log(message, level=LogLevel.ERROR)
        if self._exec_by_tool:
            exit(1)
        else:
            raise RuntimeError("RuntimeError from SDK, check the above log for details")

    def get_env_or_die(self, env_key: str) -> str:
        """Returns the value of an env variable.

        If its empty or None, raises an error and exits

        Args:
            env_key (str): Key to retrieve

        Returns:
            str: Value of the env
        """
        env_value = os.environ.get(env_key)
        if env_value is None or env_value == "":
            self.stream_error_and_exit(f"Env variable '{env_key}' is required")
        return env_value

    @staticmethod
    def stream_spec(spec: str) -> None:
        """Streams JSON schema of the tool using the Unstract protocol SPEC to
        stdout.

        Args:
            spec (str): The JSON schema of the tool.
            Typically returned by the spec() method.

        Returns:
            None
        """
        record = {
            "type": "SPEC",
            "spec": spec,
            "emitted_at": datetime.datetime.now().isoformat(),
        }
        print(json.dumps(record))

    @staticmethod
    def stream_properties(properties: str) -> None:
        """Streams the properties of the tool using the Unstract protocol
        PROPERTIES to stdout.

        Args:
            properties (str): The properties of the tool.
            Typically returned by the properties() method.
        Returns:
            None
        """
        record = {
            "type": "PROPERTIES",
            "properties": properties,
            "emitted_at": datetime.datetime.now().isoformat(),
        }
        print(json.dumps(record))

    @staticmethod
    def stream_variables(variables: str) -> None:
        """Streams JSON schema of the tool's variables using the Unstract
        protocol VARIABLES to stdout.

        Args:
            variables (str): The tool's runtime variables.
            Typically returned by the spec() method.

        Returns:
            None
        """
        record = {
            "type": Command.VARIABLES,
            "variables": variables,
            "emitted_at": datetime.datetime.now().isoformat(),
        }
        print(json.dumps(record))

    @staticmethod
    def stream_icon(icon: str) -> None:
        """Streams the icon of the tool using the Unstract protocol ICON to
        stdout.

        Args:
            icon (str): The icon of the tool.
                Typically returned by the icon() method.
        Returns:
            None
        """
        record = {
            "type": "ICON",
            "icon": icon,
            "emitted_at": datetime.datetime.now().isoformat(),
        }
        print(json.dumps(record))

    @staticmethod
    def stream_update(message: str, state: str, **kwargs: Any) -> None:
        """Streams a log message using the Unstract protocol UPDATE to stdout.

        Args:
            message (str): The log message.
            state (str): LogState from constant
        """
        record = {
            "type": "UPDATE",
            "state": state,
            "message": message,
            "emitted_at": datetime.datetime.now().isoformat(),
            **kwargs,
        }
        print(json.dumps(record))

    @staticmethod
    @deprecated(version="0.4.4", reason="Unused in workflow execution")
    def stream_cost(cost: float, cost_units: str, **kwargs: Any) -> None:
        """Streams the cost of the tool using the Unstract protocol COST to
        stdout.

        Args:
            cost (float): The cost of the tool.
            cost_units (str): The cost units of the tool.
            **kwargs: Additional keyword arguments to include in the record.
        Returns:
            None
        """
        record = {
            "type": "COST",
            "cost": cost,
            "cost_units": cost_units,
            "emitted_at": datetime.datetime.now().isoformat(),
            **kwargs,
        }
        print(json.dumps(record))

    @staticmethod
    @deprecated(version="0.4.4", reason="Unused in workflow execution")
    def stream_single_step_message(message: str, **kwargs: Any) -> None:
        """Streams a single step message using the Unstract protocol
        SINGLE_STEP_MESSAGE to stdout.

        Args:
            message (str): The single step message.
            **kwargs: Additional keyword arguments to include in the record.
        Returns:
            None
        """
        record = {
            "type": "SINGLE_STEP_MESSAGE",
            "message": message,
            "emitted_at": datetime.datetime.now().isoformat(),
            **kwargs,
        }
        print(json.dumps(record))

    @staticmethod
    @deprecated(version="0.4.4", reason="Use `BaseTool.write_to_result()` instead")
    def stream_result(result: dict[Any, Any], **kwargs: Any) -> None:
        """Streams the result of the tool using the Unstract protocol RESULT to
        stdout.

        Args:
            result (dict): The result of the tool. Refer to the
                Unstract protocol for the format of the result.
            **kwargs: Additional keyword arguments to include in the record.
        Returns:
            None
        """
        record = {
            "type": "RESULT",
            "result": result,
            "emitted_at": datetime.datetime.now().isoformat(),
            **kwargs,
        }
        print(json.dumps(record))



import logging
import warnings
from pathlib import Path

import filetype
import magic
from requests import Response
from requests.exceptions import RequestException

from unstract.sdk.adapters.constants import Common
from unstract.sdk.constants import MimeType
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class AdapterUtils:
    @staticmethod
    def get_msg_from_request_exc(
        err: RequestException,
        message_key: str,
        default_err: str = Common.DEFAULT_ERR_MESSAGE,
    ) -> str:
        """Gets the message from the RequestException.

        Args:
            err_response (Response): Error response from the exception
            message_key (str): Key from response containing error message

        Returns:
            str: Error message returned by the server
        """
        if not hasattr(err, "response"):
            return default_err

        err_response: Response = err.response  # type: ignore
        err_content_type = err_response.headers.get("Content-Type")

        if not err_content_type:
            logger.warning(
                f"Content-Type header not found in {err_response}, "
                f"returning {default_err}"
            )
            return default_err

        if err_content_type == MimeType.JSON:
            err_json = err_response.json()
            if message_key in err_json:
                return str(err_json[message_key])
            else:
                logger.warning(
                    f"Unable to parse error with key '{message_key}' for "
                    f"'{err_json}', returning '{default_err}' instead."
                )
        elif err_content_type == MimeType.TEXT:
            return err_response.text  # type: ignore
        else:
            logger.warning(
                f"Unhandled err_response type '{err_content_type}' "
                f"for {err_response}, returning {default_err}"
            )
        return default_err

    # TODO: get_file_mime_type() to be removed once migrated to FileStorage
    # FileStorage has mime_type() which could be used instead.
    @staticmethod
    def get_file_mime_type(
        input_file: Path,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        """Gets the file MIME type for an input file. Uses libmagic to perform
        the same.

        Args:
            input_file (Path): Path object of the input file

        Returns:
            str: MIME type of the file
        """
        # Adding the following DeprecationWarning manually as the package "deprecated"
        # does not support deprecation on static methods.
        warnings.warn(
            "`get_file_mime_type` is deprecated. "
            "Use `FileStorage mime_type()` instead.",
            DeprecationWarning,
        )
        sample_contents = fs.read(path=input_file, mode="rb", length=100)
        input_file_mime = magic.from_buffer(sample_contents, mime=True)
        return input_file_mime

    @staticmethod
    def guess_extention(
        input_file_path: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        """Returns the extention of the file passed.

        Args:
            input_file_path (str): String holding the path

        Returns:
            str: File extention
        """
        # Adding the following DeprecationWarning manually as the package "deprecated"
        # does not support deprecation on static methods.
        warnings.warn(
            "`guess_extention` is deprecated. "
            "Use `FileStorage guess_extension()` instead.",
            DeprecationWarning,
        )
        input_file_extention = ""
        sample_contents = fs.read(path=input_file_path, mode="rb", length=100)
        if sample_contents:
            file_type = filetype.guess(sample_contents)
            input_file_extention = file_type.EXTENSION
        return input_file_extention



from .palm import PaLMLLM

metadata = {
    "name": PaLMLLM.__name__,
    "version": "1.0.0",
    "adapter": PaLMLLM,
    "description": "Palm LLM adapter",
    "is_active": True,
}

__all__ = ["PaLMLLM"]



from typing import Any, Optional

import requests

from unstract.sdk.constants import LogLevel, ToolEnv
from unstract.sdk.helper import SdkHelper
from unstract.sdk.tool.base import BaseTool


class PlatformBase:
    """Base class to handle interactions with Unstract's platform service.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(
        self,
        tool: BaseTool,
        platform_host: str,
        platform_port: str,
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service

        Notes:
            - PLATFORM_SERVICE_API_KEY environment variable is required.
        """
        self.tool = tool
        self.base_url = SdkHelper.get_platform_base_url(platform_host, platform_port)
        self.bearer_token = tool.get_env_or_die(ToolEnv.PLATFORM_API_KEY)


class PlatformHelper(PlatformBase):
    """Implementation of `UnstractPlatformBase` to interact with platform
    service.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(self, tool: BaseTool, platform_host: str, platform_port: str):
        """Constructor of the implementation of `UnstractPlatformBase`

        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service
        """
        super().__init__(
            tool=tool, platform_host=platform_host, platform_port=platform_port
        )

    def get_platform_details(self) -> Optional[dict[str, Any]]:
        """Obtains platform details associated with the platform key.

        Currently helps fetch organization ID related to the key.

        Returns:
            Optional[dict[str, Any]]: Dictionary containing the platform details
        """
        url = f"{self.base_url}/platform_details"
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            self.tool.stream_log(
                (
                    "Error while retrieving platform details: "
                    f"[{response.status_code}] {response.reason}"
                ),
                level=LogLevel.ERROR,
            )
            return None
        else:
            platform_details: dict[str, Any] = response.json().get("details")
            return platform_details



import json
import logging
import os
import warnings
from hashlib import md5, sha256
from pathlib import Path
from typing import Any

import magic

from unstract.sdk.exceptions import FileStorageError
from unstract.sdk.file_storage import (
    FileStorage,
    FileStorageProvider,
    SharedTemporaryFileStorage,
)

logger = logging.getLogger(__name__)


class ToolUtils:
    """Class containing utility methods."""

    @staticmethod
    def hash_str(string_to_hash: Any, hash_method: str = "sha256") -> str:
        """Computes the hash for a given input string.

        Useful to hash strings needed for caching and other purposes.
        Hash method defaults to "md5"

        Args:
            string_to_hash (str): String to be hashed
            hash_method (str): Hash hash_method to use, supported ones
                - "md5"

        Returns:
            str: Hashed string
        """
        if hash_method == "md5":
            if isinstance(string_to_hash, bytes):
                return str(md5(string_to_hash).hexdigest())
            return str(md5(string_to_hash.encode()).hexdigest())
        elif hash_method == "sha256":
            if isinstance(string_to_hash, (bytes, bytearray)):
                return str(sha256(string_to_hash).hexdigest())
            return str(sha256(string_to_hash.encode()).hexdigest())
        else:
            raise ValueError(f"Unsupported hash_method: {hash_method}")

    @staticmethod
    def get_hash_from_file(
        file_path: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        """Computes the hash for a file.

        Uses sha256 to compute the file hash through a buffered read.

        Args:
            file_path (str): Path to file that needs to be hashed

        Returns:
            str: SHA256 hash of the file
        """

        # Adding the following DeprecationWarning manually as the package "deprecated"
        # does not support deprecation on static methods.
        warnings.warn(
            "`get_hash_from_file` is deprecated. "
            "Use `FileStorage get_hash_from_file()` instead.",
            DeprecationWarning,
        )
        return fs.get_hash_from_file(path=file_path)

    @staticmethod
    def load_json(
        file_to_load: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> dict[str, Any]:
        """Loads and returns a JSON from a file.

        Args:
            file_to_load (str): Path to the file containing JSON

        Returns:
            dict[str, Any]: The JSON loaded from file
        """
        file_contents: str = fs.read(path=file_to_load, mode="r", encoding="utf-8")
        loaded_json: dict[str, Any] = json.loads(file_contents)
        return loaded_json

    @staticmethod
    def dump_json(
        json_to_dump: dict[str, Any],
        file_to_dump: str,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> None:
        """Helps dump the JSON to a file.

        Args:
            json_to_dump (dict[str, Any]): Input JSON to dump
            file_to_dump (str): Path to the file to dump the JSON to
        """
        compact_json = json.dumps(json_to_dump, separators=(", ", ":"))
        fs.write(path=file_to_dump, mode="w", data=compact_json)

    @staticmethod
    def json_to_str(json_to_dump: dict[str, Any]) -> str:
        """Helps convert the JSON to a string. Useful for dumping the JSON to a
        file.

        Args:
            json_to_dump (dict[str, Any]): Input JSON to dump

        Returns:
            str: String representation of the JSON
        """
        compact_json = json.dumps(json_to_dump, separators=(",", ":"))
        return compact_json

    # ToDo: get_file_mime_type() to be removed once migrated to FileStorage
    # FileStorage has mime_type() which could be used instead.
    @staticmethod
    def get_file_mime_type(
        input_file: Path,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> str:
        """Gets the file MIME type for an input file. Uses libmagic to perform
        the same.

        Args:
            input_file (Path): Path object of the input file

        Returns:
            str: MIME type of the file
        """
        # Adding the following DeprecationWarning manually as the package "deprecated"
        # does not support deprecation on static methods.
        warnings.warn(
            "`get_file_mime_type` is deprecated. "
            "Use `FileStorage mime_type()` instead.",
            DeprecationWarning,
        )
        input_file_mime = ""
        sample_contents = fs.read(path=input_file, mode="rb", length=100)
        input_file_mime = magic.from_buffer(sample_contents, mime=True)
        return input_file_mime

    @staticmethod
    def get_file_size(
        input_file: Path,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
    ) -> int:
        """Gets the file size in bytes for an input file.
        Args:
            input_file (Path): Path object of the input file

        Returns:
            str: MIME type of the file
        """
        file_length = fs.size(path=input_file)
        return file_length

    @staticmethod
    def str_to_bool(string: str) -> bool:
        """String value of boolean to boolean.

        Useful while parsing envs to bool.

        Args:
            string (str): value like "true", "True" etc..

        Returns:
            bool
        """
        return string.lower() == "true"

    # Used the same function from LLMWhisperer
    @staticmethod
    def calculate_page_count(
        pages_string: str, max_page: int = 0, min_page: int = 1
    ) -> int:
        """Calculates the total number of pages based on the input string of
        page numbers or ranges.

        Parses the input 'pages_string' to extract individual page numbers or
        ranges separated by commas.
        Supports ranges like '1-5' or open-ended ranges like '4-'.
        The 'max_page' parameter defines the upper limit for page numbers.
        The 'min_page' parameter defines the lower limit for page numbers.

        Args:
            pages_string (str): String containing page numbers or ranges
            separated by commas
            max_page (int): Upper limit for page numbers (default is 0)
            min_page (int): Lower limit for page numbers (default is 1)

        Returns:
            int: Total count of individual pages extracted from the input string
        """
        if not pages_string:
            return max_page
        pages_list: list[int] = []
        parts = pages_string.split(",")
        for part in parts:
            part = part.strip()
            if "-" in part:
                if part.startswith("-"):  # e.g., "-5"
                    end = int(part[1:])
                    end = min(end, max_page)
                    pages_list.extend(range(min_page, end + 1))
                elif part.endswith("-"):  # e.g., "4-"
                    start = int(part[:-1])
                    if start < 0:
                        start = 0
                    if max_page is None:
                        raise ValueError(
                            "max_page must be defined for open-ended ranges like '4-'"
                        )
                    pages_list.extend(range(start, max_page + 1))
                else:  # e.g., "1-5"
                    start, end = map(int, part.split("-"))
                    if start < 0:
                        start = 0
                    if end > max_page:
                        end = max_page
                    pages_list.extend(range(start, end + 1))
            else:
                pages_list.append(int(part))
        return len(pages_list)

    @staticmethod
    def get_filestorage_provider(
        var_name: str, default: str = "minio"
    ) -> FileStorageProvider:
        """Retrieve the file storage provider based on an environment
        variable."""
        provider_name = os.environ.get(var_name, default).upper()
        try:
            # Attempt to map the provider name to an enum value, case-insensitively
            return FileStorageProvider[provider_name]
        except KeyError:
            allowed_providers = ", ".join(
                [provider.name for provider in FileStorageProvider]
            )
            logger.error(
                f"Invalid provider '{provider_name}'. Allowed providers: "
                f"{allowed_providers}"
            )
            raise FileStorageError(f"Invalid provider '{provider_name}'")

    @staticmethod
    def get_filestorage_credentials(var_name: str) -> dict[str, Any]:
        """Retrieve the file storage credentials based on an environment
        variable."""
        credentials = os.environ.get(var_name, "{}")
        try:
            return json.loads(credentials)
        except json.JSONDecodeError:
            raise ValueError(
                "File storage credentials are not set properly. "
                "Please check your settings."
            )

    @staticmethod
    def get_workflow_filestorage(
        provider: FileStorageProvider,
        credentials: dict[str, Any] = {},
    ) -> SharedTemporaryFileStorage:
        """Get the file storage for the workflow."""
        return SharedTemporaryFileStorage(provider=provider, **credentials)



from dataclasses import dataclass
from typing import Optional


@dataclass
class TextExtractionMetadata:
    whisper_hash: str


@dataclass
class TextExtractionResult:
    extracted_text: str
    extraction_metadata: Optional[TextExtractionMetadata] = None



import logging

from unstract.sdk.constants import PublicAdapterKeys

logger = logging.getLogger(__name__)

class SdkHelper:
    def __init__(self) -> None:
        pass

    @staticmethod
    def get_platform_base_url(platform_host: str, platform_port: str) -> str:
        """Make base url from host and port.

        Args:
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service

        Returns:
            str: URL to the platform service
        """
        if platform_host[-1] == "/":
            return f"{platform_host[:-1]}:{platform_port}"
        return f"{platform_host}:{platform_port}"

    @staticmethod
    def is_public_adapter(adapter_id: str) -> bool:
        """Check if the given adapter_id is one of the public adapter keys.

        This method iterates over the attributes of the PublicAdapterKeys class
        and checks if the provided adapter_id matches any of the attribute values.

        Args:
            adapter_id (str): The ID of the adapter to check.

        Returns:
            bool: True if the adapter_id matches any public adapter key,
            False otherwise.
        """
        try:
            for attr in dir(PublicAdapterKeys):
                if getattr(PublicAdapterKeys, attr) == adapter_id:
                    return True
            return False
        except Exception as e:
            logger.warning(
                f"Unable to determine if adapter_id: {adapter_id}"
                f"is public or not: {str(e)}"
            )
            return False



import os
from typing import Any

from llama_index.core.llms import LLM
from llama_index.llms.openai import OpenAI
from llama_index.llms.openai.utils import O1_MODELS
from openai import APIError as OpenAIAPIError

from unstract.sdk.adapters.exceptions import AdapterError
from unstract.sdk.adapters.llm.constants import LLMKeys
from unstract.sdk.adapters.llm.llm_adapter import LLMAdapter
from unstract.sdk.exceptions import LLMError

class Constants:
    MODEL = "model"
    API_KEY = "api_key"
    MAX_RETRIES = "max_retries"
    ADAPTER_NAME = "adapter_name"
    TIMEOUT = "timeout"
    API_BASE = "api_base"
    API_VERSION = "api_version"
    MAX_TOKENS = "max_tokens"


class OpenAILLM(LLMAdapter):
    
    def __init__(self, settings: dict[str, Any]):
        super().__init__("OpenAI")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "openai|502ecf49-e47c-445c-9907-6d4b90c5cd17"

    @staticmethod
    def get_name() -> str:
        return "OpenAI"

    @staticmethod
    def get_description() -> str:
        return "OpenAI LLM"

    @staticmethod
    def get_provider() -> str:
        return "openai"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/OpenAI.png"

    def get_llm_instance(self) -> LLM:
        try:
            max_tokens = self.config.get(Constants.MAX_TOKENS)
            max_tokens = int(max_tokens) if max_tokens else None
            model = str(self.config.get(Constants.MODEL))

            llm_kwargs = {
                "model": model,
                "api_key": str(self.config.get(Constants.API_KEY)),
                "api_base": str(self.config.get(Constants.API_BASE)),
                "api_version": str(self.config.get(Constants.API_VERSION)),
                "max_retries": int(self.config.get(Constants.MAX_RETRIES, LLMKeys.DEFAULT_MAX_RETRIES)),
                "api_type": "openai",
                "timeout": float(self.config.get(Constants.TIMEOUT, LLMKeys.DEFAULT_TIMEOUT)),
                "max_tokens": max_tokens,
            }

            # O-series models default to temperature=1, ignoring passed values, so it's not set explicitly.
            if model not in O1_MODELS:
                llm_kwargs["temperature"] = 0

            llm = OpenAI(**llm_kwargs)
            return llm
        except Exception as e:
            raise AdapterError(str(e))

    @staticmethod
    def parse_llm_err(e: OpenAIAPIError) -> LLMError:
        """Parse the error from OpenAI.

        Helps parse errors from OpenAI and wraps with custom exception.

        Args:
            e (OpenAIAPIError): Exception from OpenAI

        Returns:
            LLMError: Error to be sent to the user
        """
        if hasattr(e, "body") and isinstance(e.body, dict) and "message" in e.body:
            msg = e.body["message"]
        else:
            msg = e.message

        status_code = e.status_code if hasattr(e, "status_code") else None
        return LLMError(msg, actual_err=e, status_code=status_code)



import argparse
from typing import Optional

from dotenv import find_dotenv, load_dotenv

from unstract.sdk.constants import LogLevel


class ToolArgsParser:
    """Class to help with parsing arguments to a tool."""

    @staticmethod
    def parse_args(args_to_parse: list[str]) -> argparse.Namespace:
        """Helps parse arguments to a tool.

        Args:
            args_to_parse (List[str]): Command line arguments received by a tool

        Returns:
            argparse.Namespace: Parsed arguments
        """
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "--command", type=str, help="Command to execute", required=True
        )
        parser.add_argument(
            "--settings", type=str, help="Settings to be used", required=False
        )
        parser.add_argument(
            "--log-level",
            type=LogLevel,
            help="Log level",
            required=False,
            default=LogLevel.ERROR,
        )
        parser.add_argument(
            "--env",
            type=str,
            help="Env file to load environment from",
            required=False,
            default=find_dotenv(usecwd=True),
        )
        parsed_args = parser.parse_args(args_to_parse)
        ToolArgsParser.load_environment(parsed_args.env)
        return parsed_args

    @staticmethod
    def load_environment(path: Optional[str] = None) -> None:
        """Loads env variables with python-dotenv.

        Args:
            path (Optional[str], optional): Path to the env file to load.
                Defaults to None.
        """
        if path is None:
            load_dotenv()
        else:
            load_dotenv(path)



from unstract.sdk.adapters.vectordb.no_op.src.no_op_vectordb import NoOpVectorDB

metadata = {
    "name": NoOpVectorDB.__name__,
    "version": "1.0.0",
    "adapter": NoOpVectorDB,
    "description": "NoOpVectorDB",
    "is_active": True,
}



import json
import logging
import os
from typing import Any, Optional

import requests

from unstract.sdk.adapters.x2text.constants import X2TextConstants
from unstract.sdk.adapters.x2text.dto import (
    TextExtractionMetadata,
    TextExtractionResult,
)
from unstract.sdk.adapters.x2text.llm_whisperer_v2.src.constants import (
    HTTPMethod,
    WhispererEndpoint,
)
from unstract.sdk.adapters.x2text.llm_whisperer_v2.src.dto import WhispererRequestParams
from unstract.sdk.adapters.x2text.llm_whisperer_v2.src.helper import LLMWhispererHelper
from unstract.sdk.adapters.x2text.x2text_adapter import X2TextAdapter
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class LLMWhispererV2(X2TextAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("LLMWhispererV2")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "llmwhisperer|a5e6b8af-3e1f-4a80-b006-d017e8e67f93"

    @staticmethod
    def get_name() -> str:
        return "LLMWhisperer V2"

    @staticmethod
    def get_description() -> str:
        return "LLMWhisperer V2 X2Text"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/LLMWhispererV2.png"

    def test_connection(self) -> bool:
        LLMWhispererHelper.test_connection_request(
            config=self.config,
            request_endpoint=WhispererEndpoint.TEST_CONNECTION,
        )
        return True

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        **kwargs: dict[Any, Any],
    ) -> TextExtractionResult:
        """Used to extract text from documents.

        Args:
            input_file_path (str): Path to file that needs to be extracted
            output_file_path (Optional[str], optional): File path to write
                extracted text into, if None doesn't write to a file.
                Defaults to None.

        Returns:
            str: Extracted text
        """

        extra_params = WhispererRequestParams(tag=kwargs.get(X2TextConstants.TAGS))
        response: requests.Response = LLMWhispererHelper.send_whisper_request(
            input_file_path=input_file_path,
            config=self.config,
            fs=fs,
            extra_params=extra_params,
        )
        metadata = TextExtractionMetadata(
            whisper_hash=response.get(X2TextConstants.WHISPER_HASH_V2, "")
        )

        return TextExtractionResult(
            extracted_text=LLMWhispererHelper.extract_text_from_response(
                output_file_path, response, fs=fs
            ),
            extraction_metadata=metadata,
        )



import logging
import os
import time
from typing import Any, Optional

from unstract.sdk.adapters.x2text.dto import TextExtractionResult
from unstract.sdk.adapters.x2text.x2text_adapter import X2TextAdapter
from unstract.sdk.file_storage import FileStorage, FileStorageProvider

logger = logging.getLogger(__name__)


class NoOpX2Text(X2TextAdapter):
    def __init__(self, settings: dict[str, Any]):
        super().__init__("NoOpX2Text")
        self.config = settings

    SCHEMA_PATH = f"{os.path.dirname(__file__)}/static/json_schema.json"

    @staticmethod
    def get_id() -> str:
        return "noOpX2text|mp66d1op-7100-d340-9101-846fc7115676"

    @staticmethod
    def get_name() -> str:
        return "No Op X2Text"

    @staticmethod
    def get_description() -> str:
        return "No Op X2Text Adapter"

    @staticmethod
    def get_icon() -> str:
        return "/icons/adapter-icons/noOpx2Text.png"

     

    def process(
        self,
        input_file_path: str,
        output_file_path: Optional[str] = None,
        fs: FileStorage = FileStorage(provider=FileStorageProvider.LOCAL),
        **kwargs: dict[Any, Any],
    ) -> TextExtractionResult:
        extracted_text: str = (
            "This is a No Op x2text adapter response."
            " This is a sample response and intended for testing \f"
        )
        time.sleep(self.config.get("wait_time"))
        if output_file_path:
            fs.write(
                path=output_file_path, mode="w", data=extracted_text, encoding="utf-8"
            )
        return TextExtractionResult(extracted_text=extracted_text)

    def test_connection(self) -> bool:
        time.sleep(self.config.get("wait_time"))
        return True



import logging
from typing import Any, Optional

import requests
from requests import ConnectionError, RequestException, Response

from unstract.sdk.constants import LogLevel, MimeType, PromptStudioKeys, ToolEnv
from unstract.sdk.helper import SdkHelper
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.utils.common_utils import log_elapsed

logger = logging.getLogger(__name__)


class PromptTool:
    """Class to handle prompt service methods for Unstract Tools."""

    def __init__(
        self,
        tool: BaseTool,
        prompt_host: str,
        prompt_port: str,
        is_public_call: bool = False,
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            prompt_host (str): Host of platform service
            prompt_host (str): Port of platform service
        """
        self.tool = tool
        self.base_url = SdkHelper.get_platform_base_url(prompt_host, prompt_port)
        self.is_public_call = is_public_call
        if not is_public_call:
            self.bearer_token = tool.get_env_or_die(ToolEnv.PLATFORM_API_KEY)

    @log_elapsed(operation="ANSWER_PROMPTS")
    def answer_prompt(
        self, payload: dict[str, Any], params: Optional[dict[str, str]] = None
    ) -> dict[str, Any]:
        url_path = "answer-prompt"
        if self.is_public_call:
            url_path = "answer-prompt-public"
        return self._post_call(
            url_path=url_path,
            payload=payload,
            params=params,
        )

    def single_pass_extraction(
        self, payload: dict[str, Any], params: Optional[dict[str, str]] = None
    ) -> dict[str, Any]:
        return self._post_call(
            url_path="single-pass-extraction",
            payload=payload,
            params=params,
        )

    def summarize(
        self, payload: dict[str, Any], params: Optional[dict[str, str]] = None
    ) -> dict[str, Any]:
        return self._post_call(url_path="summarize", payload=payload, params=params)

    def _post_call(
        self,
        url_path: str,
        payload: dict[str, Any],
        params: Optional[dict[str, str]] = None,
    ) -> dict[str, Any]:
        """Invokes and communicates to prompt service to fetch response for the
        prompt.

        Args:
            url_path (str): URL path to the service endpoint
            payload (dict): Payload to send in the request body
            params (dict, optional): Query parameters to include in the request

        Returns:
            dict: Response from the prompt service

            Sample Response:
            {
                "status": "OK",
                "error": "",
                "cost": 0,
                structure_output : {}
            }
        """
        result: dict[str, Any] = {
            "status": "ERROR",
            "error": "",
            "cost": 0,
            "structure_output": "",
            "status_code": 500,
        }
        url: str = f"{self.base_url}/{url_path}"
        headers: dict[str, str] = {}
        if not self.is_public_call:
            headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response: Response = Response()
        try:
            response = requests.post(
                url=url, json=payload, params=params, headers=headers
            )
            response.raise_for_status()
            result["status"] = "OK"
            result["structure_output"] = response.text
            result["status_code"] = 200
        except ConnectionError as connect_err:
            msg = "Unable to connect to prompt service. Please contact admin."
            self._stringify_and_stream_err(connect_err, msg)
            result["error"] = msg
        except RequestException as e:
            # Extract error information from the response if available
            error_message = str(e)
            content_type = response.headers.get("Content-Type", "").lower()
            if MimeType.JSON in content_type:
                response_json = response.json()
                if "error" in response_json:
                    error_message = response_json["error"]
            elif response.text:
                error_message = response.text
            result["error"] = error_message
            result["status_code"] = response.status_code
            self.tool.stream_log(
                f"Error while fetching response for prompt: {result['error']}",
                level=LogLevel.ERROR,
            )
        return result

    def _stringify_and_stream_err(self, err: RequestException, msg: str) -> None:
        error_message = str(err)
        trace = f"{msg}: {error_message}"
        self.tool.stream_log(trace, level=LogLevel.ERROR)
        logger.error(trace)

    @staticmethod
    def get_exported_tool(
        tool: BaseTool, prompt_registry_id: str
    ) -> Optional[dict[str, Any]]:
        """Get exported custom tool by the help of unstract DB tool.

        Args:
            prompt_registry_id (str): ID of the prompt_registry_id
            tool (AbstractTool): Instance of AbstractTool
        Required env variables:
            PLATFORM_HOST: Host of platform service
            PLATFORM_PORT: Port of platform service
        """
        platform_host = tool.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = tool.get_env_or_die(ToolEnv.PLATFORM_PORT)
        base_url = SdkHelper.get_platform_base_url(platform_host, platform_port)
        bearer_token = tool.get_env_or_die(ToolEnv.PLATFORM_API_KEY)
        url = f"{base_url}/custom_tool_instance"
        query_params = {PromptStudioKeys.PROMPT_REGISTRY_ID: prompt_registry_id}
        headers = {"Authorization": f"Bearer {bearer_token}"}
        response = requests.get(url, headers=headers, params=query_params)
        if response.status_code == 200:
            return response.json()
        elif response.status_code == 404:
            tool.stream_error_and_exit(
                f"Exported tool '{prompt_registry_id}' is not found"
            )
            return None
        else:
            tool.stream_error_and_exit(
                f"Error while retrieving tool metadata "
                "for the exported tool: "
                f"{prompt_registry_id} / {response.reason}"
            )
            return None



class FileType:
    TEXT_PLAIN = "text/plain"
    IMAGE_JPEG = "image/jpeg"
    IMAGE_PNG = "image/png"
    IMAGE_TIFF = "image/tiff"
    IMAGE_BMP = "image/bmp"
    IMAGE_GIF = "image/gif"
    IMAGE_WEBP = "image/webp"
    APPLICATION_PDF = "application/pdf"
    ALLOWED_TYPES = [
        IMAGE_JPEG,
        IMAGE_PNG,
        IMAGE_TIFF,
        IMAGE_BMP,
        IMAGE_GIF,
        IMAGE_WEBP,
        APPLICATION_PDF,
    ]



from abc import ABC, abstractmethod
from typing import Any

from llama_index.core import MockEmbedding
from llama_index.core.embeddings import BaseEmbedding

from unstract.sdk.adapters.base import Adapter
from unstract.sdk.adapters.enums import AdapterTypes

from unstract.sdk.adapters.embedding.helper import EmbeddingHelper

class EmbeddingAdapter(Adapter, ABC):
    def __init__(self, name: str):
        super().__init__(name)
        self.name = name

    @staticmethod
    def get_id() -> str:
        return ""

    @staticmethod
    def get_name() -> str:
        return ""

    @staticmethod
    def get_description() -> str:
        return ""

    @staticmethod
    @abstractmethod
    def get_provider() -> str:
        pass

    @staticmethod
    def get_icon() -> str:
        return ""

    @staticmethod
    def get_adapter_type() -> AdapterTypes:
        return AdapterTypes.EMBEDDING

    def get_embedding_instance(self, embed_config: dict[str, Any]) -> BaseEmbedding:
        """Instantiate the llama index BaseEmbedding class.

        Returns:
            BaseEmbedding: llama index implementation of the Embedding
            Raises exceptions for any error
        """
        return MockEmbedding(embed_dim=1)
    
    def test_connection(self) -> bool:
        embedding = self.get_embedding_instance()
        test_result: bool = EmbeddingHelper.test_embedding_instance(embedding)
        return test_result


import json
import logging
from datetime import datetime
from hashlib import sha256
from typing import Any, Union

import filetype
import fsspec
import magic
import yaml

from unstract.sdk.exceptions import FileOperationError
from unstract.sdk.file_storage.constants import FileOperationParams, FileSeekPosition
from unstract.sdk.file_storage.helper import FileStorageHelper, skip_local_cache
from unstract.sdk.file_storage.interface import FileStorageInterface
from unstract.sdk.file_storage.provider import FileStorageProvider

logger = logging.getLogger(__name__)


class FileStorage(FileStorageInterface):
    # This class integrates fsspec library for file operations

    fs: fsspec  # fsspec file system handle
    provider: FileStorageProvider

    def __init__(self, provider: FileStorageProvider, **storage_config: dict[str, Any]):
        self.fs = FileStorageHelper.file_storage_init(provider, **storage_config)
        self.provider = provider

    @skip_local_cache
    def read(
        self,
        path: str,
        mode: str,
        encoding: str = FileOperationParams.DEFAULT_ENCODING,
        seek_position: int = 0,
        length: int = FileOperationParams.READ_ENTIRE_LENGTH,
    ) -> Union[bytes, str]:
        """Read the file pointed to by the file_handle.

        Args:
            path (str): Path to the file to be opened
            mode (str): Mode in which the file is to be opened. Usual options
                        include r, rb, w and wb
            encoding (str): Encoding type like utf-8 or utf-16
            seek_position (int): Position to start reading from
            length (int): Number of bytes to be read. Default is full
            file content.

        Returns:
            Union[bytes, str] - File contents in bytes/string based on the opened mode
        """
        with self.fs.open(path=path, mode=mode, encoding=encoding) as file_handle:
            if seek_position > 0:
                file_handle.seek(seek_position)
            return file_handle.read(length)

    def write(
        self,
        path: str,
        mode: str,
        encoding: str = FileOperationParams.DEFAULT_ENCODING,
        seek_position: int = 0,
        data: Union[bytes, str] = "",
    ) -> int:
        """Write data in the file pointed to by the file-handle.

        Args:
            path (str): Path to the file to be opened
            mode (str): Mode in whicg the file is to be opened. Usual options
                        include r, rb, w and wb
            encoding (str): Encoding type like utf-8 or utf-16
            seek_position (int): Position to start writing from
            data (Union[bytes, str]): Contents to be written

        Returns:
            int: Number of bytes that were successfully written to the file
        """
        try:
            with self.fs.open(path=path, mode=mode, encoding=encoding) as file_handle:
                return file_handle.write(data)
        except Exception as e:
            raise FileOperationError(str(e)) from e

    @skip_local_cache
    def seek(
        self,
        path: str,
        location: int = 0,
        position: FileSeekPosition = FileSeekPosition.START,
    ) -> int:
        """Place the file pointer to the mentioned location in the file
        relative to the position.

        Args:
            path (str): path of the file
            location (int): Nth byte position. To be understood in relation to
            the arg "position"
            position (FileSeekPosition): from start of file, current location
            or end of file

        Returns:
            int: file pointer location after seeking to the mentioned position
        """
        with self.fs.open(path=path, mode="rb") as file_handle:
            return file_handle.seek(location, position)

    def mkdir(self, path: str, create_parents: bool = True):
        """Create a directory.

        Args:
            path (str): Path of the directory to be created
            create_parents (bool): Specify if parent directories to be created
            if any of the nested directory does not exist
        """
        try:
            self.fs.mkdir(path=path, create_parents=create_parents)
        except FileExistsError:
            logger.debug(f"Path {path} already exists.")
        except Exception as e:
            raise FileOperationError(str(e)) from e

    @skip_local_cache
    def exists(self, path: str) -> bool:
        """Checks if a file/directory path exists.

        Args:
            path (str): File/directory path

        Returns:
            bool: If the file/directory  exists or not
        """
        try:
            return self.fs.exists(path)
        except Exception as e:
            raise FileOperationError(str(e)) from e

    @skip_local_cache
    def ls(self, path: str) -> list[str]:
        """List the directory path.

        Args:
            path (str): Directory path

        Returns:
            List[str]: List of files / directories under the path
        """
        return self.fs.ls(path)

    @skip_local_cache
    def rm(self, path: str, recursive: bool = True):
        """Removes a file or directory mentioned in path.

        Args:
            path (str): Path to the file / directory
            recursive (bool): Whether the files and folders nested
            under path are to be removed or not

        Returns:
            NA
        """
        return self.fs.rm(path=path, recursive=recursive)

    @skip_local_cache
    def cp(
        self,
        src: str,
        dest: str,
        recursive: bool = False,
        overwrite: bool = True,
    ):
        """Copies files from source(lpath) path to the destination(rpath) path.

        Args:
            src (str): Path to the source
            dest (str): Path to the destination
            recursive (bool): Copy recursively when set to True
            overwrite (bool): Overwrite existing path with same name

        Returns:
            NA
        """
        return self.fs.cp(src, dest, recursive=recursive, overwrite=overwrite)

    @skip_local_cache
    def size(self, path: str) -> int:
        """Get the size of the file specified in path.

        Args:
            path (str): Path to the file

        Returns:
            int: Size of the file in bytes
        """
        file_info = self.fs.info(path)
        return file_info["size"]

    @skip_local_cache
    def modification_time(self, path: str) -> datetime:
        """Get the last modification time of the file specified in path.

        Args:
            path (str): Path to the file

        Returns:
            datetime: Last modified time in datetime
        """
        file_info = self.fs.info(path)
        file_mtime = file_info["mtime"]
        if not isinstance(file_mtime, datetime):
            file_mtime = datetime.fromtimestamp(file_mtime)
        return file_mtime

    def mime_type(
        self,
        path: str,
        read_length: int = FileOperationParams.READ_ENTIRE_LENGTH,
    ) -> str:
        """Gets the file MIME type for an input file. Uses libmagic to perform
        the same.

        Args:
            path (str): Path of the input file
            read_length (int): Length(bytes) to be read from the file for in
            order to identify the mime type. Defaults to read the entire length.

        Returns:
            str: MIME type of the file
        """
        sample_contents = self.read(path=path, mode="rb", length=read_length)
        mime_type = magic.from_buffer(sample_contents, mime=True)
        return mime_type

    @skip_local_cache
    def download(self, from_path: str, to_path: str):
        """Downloads the file mentioned in from_path to to_path on the local
        system. The instance calling the method needs to be the FileStorage
        initialised with the remote file system.

        Args:
            from_path (str): Path of the file to be downloaded (remote)
            to_path (str): Path where the file is to be downloaded
            on local system

        Returns:
            NA
        """
        self.fs.get(rpath=from_path, lpath=to_path)

    @skip_local_cache
    def upload(self, from_path: str, to_path: str):
        """Uploads the file mentioned in from_path (local system) to to_path
        (remote system). The instance calling the method needs to be the
        FileStorage initialised with the remote file system where the file
        needs to be uploaded.

        Args:
            from_path (str): Path of the file to be uploaded (local)
            to_path (str): Path where the file is to be uploaded (usually remote)

        Returns:
            NA
        """
        self.fs.put(from_path, to_path)

    def glob(self, path: str) -> list[str]:
        """Lists files under path matching the pattern sepcified as part of
        path in the argument.

        Args:
            path (str): path to the directory where files matching the
            specified pattern is to be found
            Eg. a/b/c/*.txt will list all txt files under a/b/c/

        Returns:
            list[str]: List of file names matching any pattern specified
        """
        try:
            return self.fs.glob(path)
        except Exception as e:
            raise FileOperationError(str(e)) from e

    @skip_local_cache
    def get_hash_from_file(self, path: str) -> str:
        """Computes the hash for a file.

        Uses sha256 to compute the file hash through a buffered read.

        Args:
            file_path (str): Path to file that needs to be hashed

        Returns:
            str: SHA256 hash of the file
        """

        h = sha256()
        b = bytearray(128 * 1024)
        mv = memoryview(b)
        with self.fs.open(path) as f:
            while n := f.readinto(mv):
                h.update(mv[:n])
        return str(h.hexdigest())

    def json_dump(
        self,
        path: str,
        data: dict[str, Any],
        **kwargs: dict[Any, Any],  # type: ignore
    ):
        """Dumps data into the given file specified by path.

        Args:
            path (str): Path to file where JSON is to be dumped
            data (dict): Object to be written to the file
            **kwargs (dict): Any other additional arguments
        """
        try:
            with self.fs.open(path=path, mode="w", encoding="utf-8") as f:
                json.dump(obj=data, fp=f, **kwargs)  # type: ignore
        except Exception as e:
            raise FileOperationError(str(e)) from e

    def yaml_dump(
        self,
        path: str,
        data: dict[str, Any],
        **kwargs: dict[Any, Any],  # type: ignore
    ):
        """Dumps data into the given file specified by path.

        Args:
            path (str): Path to file where yml is to be dumped
            data (dict): Object to be written to the file
            **kwargs (dict): Any other additional arguments
        """
        try:
            with self.fs.open(path=path, mode="w", encoding="utf-8") as f:
                yaml.dump(data=data, stream=f, **kwargs)  # type: ignore
        except Exception as e:
            raise FileOperationError(str(e)) from e

    @skip_local_cache
    def json_load(self, path: str) -> dict[Any, Any]:
        with self.fs.open(path=path) as json_file:
            data: dict[str, Any] = json.load(json_file)
            return data

    @skip_local_cache
    def yaml_load(
        self,
        path: str,
    ) -> dict[Any, Any]:
        """Loads data from a file as yaml.

        Args:
            path (str): Path to file where yml is to be loaded

        Returns:
            dict[Any, Any]: Data loaded as yaml
        """
        with self.fs.open(path=path) as f:
            data: dict[str, Any] = yaml.safe_load(f)
            return data

    def guess_extension(self, path: str) -> str:
        """Returns the extension of the file passed.

        Args:
            path (str): String holding the path

        Returns:
            str: File extension
        """
        file_extension = ""
        sample_contents = self.read(
            path=path,
            mode="rb",
            length=FileOperationParams.EXTENSION_DEFAULT_READ_LENGTH,
        )
        if sample_contents:
            file_type = filetype.guess(sample_contents)
            file_extension = file_type.EXTENSION
        return file_extension

    def walk(self, path: str, max_depth=None, topdown=True):
        """Walks the dir in the path and returns the list of files/dirs.

        Args:
            path (str): Root to recurse into
            maxdepth (int): Maximum recursion depth. None means limitless,
            but not recommended
                on link-based file-systems.
            topdown (bool): Whether to walk the directory tree from the top
            downwards or from
                the bottom upwards.

        Returns:
            Iterator containing the list of files and folders
        """
        # Invalidating cache explicitly to avoid any stale listing
        self.fs.invalidate_cache(path=path)
        return self.fs.walk(path, maxdepth=max_depth, topdown=topdown)



import io
import json
import os.path
from json import JSONDecodeError

import pdfplumber
import pytest
from dotenv import load_dotenv

from unstract.sdk.constants import MimeType
from unstract.sdk.exceptions import FileOperationError
from unstract.sdk.file_storage.constants import FileOperationParams, StorageType
from unstract.sdk.file_storage.env_helper import EnvHelper
from unstract.sdk.file_storage.impl import FileStorage
from unstract.sdk.file_storage.provider import FileStorageProvider

load_dotenv()


class TEST_CONSTANTS:
    READ_FOLDER_PATH = os.environ.get("READ_FOLDER_PATH")
    WRITE_FOLDER_PATH = os.environ.get("WRITE_FOLDER_PATH")
    RECURSION_FOLDER_PATH = os.environ.get("RECURSION_FOLDER_PATH")
    READ_PDF_FILE = os.environ.get("READ_PDF_FILE")
    READ_TEXT_FILE = os.environ.get("READ_TEXT_FILE")
    WRITE_PDF_FILE = os.environ.get("WRITE_PDF_FILE")
    WRITE_TEXT_FILE = os.environ.get("WRITE_TEXT_FILE")
    TEST_FOLDER = os.environ.get("TEST_FOLDER")
    GCS_BUCKET = os.environ.get("GCS_BUCKET")
    TEXT_CONTENT = os.environ.get("TEXT_CONTENT")
    FILE_STORAGE_GCS = "FILE_STORAGE_GCS"
    FILE_STORAGE_MINIO = "FILE_STORAGE_MINIO"
    FILE_STORAGE_LOCAL = "FILE_STORAGE_LOCAL"
    FILE_STORAGE_S3 = "FILE_STORAGE_S3"
    FILE_STORAGE_AZURE = "FILE_STORAGE_AZURE"


def file_storage(provider: FileStorageProvider):
    try:
        if provider == FileStorageProvider.GCS:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_GCS, "{}"))
        elif provider == FileStorageProvider.MINIO:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_MINIO, "{}"))
        elif provider == FileStorageProvider.LOCAL:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_LOCAL, "{}"))
        elif provider == FileStorageProvider.S3:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_S3, "{}"))
        elif provider == FileStorageProvider.AZURE:
            creds = json.loads(os.environ.get(TEST_CONSTANTS.FILE_STORAGE_AZURE, "{}"))
    except JSONDecodeError:
        creds = {}
    file_storage = FileStorage(provider, **creds)
    assert file_storage is not None
    return file_storage


@pytest.mark.parametrize(
    "file_storage, path, mode, read_length, expected_read_length",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            "rb",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "r",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "r",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.S3),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            0,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            0,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
    ],
)
def test_file_read(file_storage, path, mode, read_length, expected_read_length):
    file_contents = file_storage.read(path=path, mode=mode, length=read_length)
    assert len(file_contents) == expected_read_length


@pytest.mark.parametrize(
    "file_storage, path, mode, read_length",
    [
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            "1.txt",
            "rb",
            -1,
        ),
    ],
)
def test_file_read_exception(file_storage, path, mode, read_length):
    with pytest.raises(FileNotFoundError):
        file_storage.read(path=path, mode=mode, length=read_length)


@pytest.mark.parametrize(
    "file_storage, read_file_path, read_mode, file_contents, "
    "write_file_path, write_mode, read_length, expected_write_length",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_PDF_FILE,
            "wb",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_PDF_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_PDF_FILE,
            "wb",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            None,
            "rb",
            TEST_CONSTANTS.TEXT_CONTENT,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "w",
            0,
            len(TEST_CONSTANTS.TEXT_CONTENT),
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            None,
            "rb",
            TEST_CONSTANTS.TEXT_CONTENT.encode(),
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            len(TEST_CONSTANTS.TEXT_CONTENT.encode()),
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_PDF_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_PDF_FILE,
            "wb",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            None,
            "rb",
            TEST_CONSTANTS.TEXT_CONTENT,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "w",
            0,
            len(TEST_CONSTANTS.TEXT_CONTENT),
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_PDF_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_PDF_FILE,
            "wb",
            -1,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            None,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "wb",
            0,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            None,
            "rb",
            TEST_CONSTANTS.TEXT_CONTENT,
            TEST_CONSTANTS.WRITE_TEXT_FILE,
            "w",
            0,
            len(TEST_CONSTANTS.TEXT_CONTENT),
        ),
    ],
)
def test_file_write(
    file_storage,
    read_file_path,
    read_mode,
    file_contents,
    write_file_path,
    write_mode,
    read_length,
    expected_write_length,
):
    if file_storage.exists(path=TEST_CONSTANTS.WRITE_FOLDER_PATH):
        file_storage.rm(path=TEST_CONSTANTS.WRITE_FOLDER_PATH, recursive=True)
    assert file_storage.exists(path=TEST_CONSTANTS.WRITE_FOLDER_PATH) is False
    file_storage.mkdir(path=TEST_CONSTANTS.WRITE_FOLDER_PATH)
    # assert file_storage.path_exists(path=TEST_CONSTANTS.WRITE_FOLDER_PATH) == True
    if read_file_path:
        file_contents = file_storage.read(
            path=read_file_path, mode=read_mode, length=read_length
        )
    else:
        file_contents = file_contents
    bytes_written = file_storage.write(
        path=write_file_path, mode=write_mode, data=file_contents
    )
    assert bytes_written == expected_write_length


@pytest.mark.parametrize(
    "file_storage, folder_path, expected_result",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.TEST_FOLDER,
            False,  # mkdir does not work for blob storages
            # as they only support creating buckets. For
            # further details pls check implementation of mkdir in GCSFS
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.TEST_FOLDER,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.TEST_FOLDER,
            False,  # mkdir does not work for blob storages
            # as they only support creating buckets. For
            # further details pls check implementation of mkdir in S3
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.TEST_FOLDER,
            False,
        ),
    ],
)
def test_make_dir(file_storage, folder_path, expected_result):
    if file_storage.exists(path=folder_path):
        file_storage.rm(path=folder_path, recursive=True)
    file_storage.mkdir(folder_path)
    assert file_storage.exists(path=folder_path) == expected_result


@pytest.mark.parametrize(
    "file_storage, folder_path, expected_result",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.GCS_BUCKET,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.TEST_FOLDER,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            "dummy",
            False,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.GCS_BUCKET,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.GCS_BUCKET,
            True,
        ),
    ],
)
def test_path_exists(file_storage, folder_path, expected_result):
    assert file_storage.exists(path=folder_path) == expected_result


@pytest.mark.parametrize(
    "file_storage, folder_path, expected_file_count",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_FOLDER_PATH,
            3,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_FOLDER_PATH,
            3,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            1,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_FOLDER_PATH,
            2,
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_FOLDER_PATH,
            2,
        ),
    ],
)
def test_ls(file_storage, folder_path, expected_file_count):
    assert len(file_storage.ls(folder_path)) == expected_file_count


@pytest.mark.parametrize(
    "file_storage, folder_path",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.WRITE_FOLDER_PATH,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.WRITE_FOLDER_PATH,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.WRITE_FOLDER_PATH,
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.WRITE_FOLDER_PATH,
        ),
    ],
)
def test_rm(file_storage, folder_path):
    if not file_storage.exists(path=folder_path):
        file_storage.mkdir(path=folder_path)
        if not file_storage.exists(path=folder_path):
            file_storage.fs.touch(path=folder_path)
    file_storage.rm(path=folder_path)
    assert file_storage.exists(path=folder_path) is False


@pytest.mark.parametrize(
    "file_storage, folder_path, recursive, test_folder_path",
    [
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.RECURSION_FOLDER_PATH,
            True,
            TEST_CONSTANTS.WRITE_FOLDER_PATH,
        ),
    ],
)
def test_rm_recursive(file_storage, folder_path, recursive, test_folder_path):
    if not file_storage.exists(path=folder_path):
        file_storage.mkdir(path=folder_path)
    assert file_storage.exists(path=folder_path) is True
    file_storage.rm(path=test_folder_path, recursive=recursive)
    assert file_storage.exists(path=test_folder_path) is False


@pytest.mark.parametrize(
    "file_storage, folder_path, recursive, test_folder_path",
    [
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.RECURSION_FOLDER_PATH,
            False,
            TEST_CONSTANTS.WRITE_FOLDER_PATH,
        ),
    ],
)
def test_rm_recursive_exception(file_storage, folder_path, recursive, test_folder_path):
    if not file_storage.exists(path=folder_path):
        file_storage.mkdir(path=folder_path)
    assert file_storage.exists(path=folder_path) is True
    with pytest.raises(FileOperationError):
        file_storage.rm(path=test_folder_path, recursive=recursive)


@pytest.mark.parametrize(
    "file_storage, file_path, mode, location, whence, expected_size",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            1,
            0,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE) - 1,
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            "r",
            0,
            2,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            1,
            0,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE) - 1,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_PDF_FILE,
            "r",
            0,
            2,
            0,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "rb",
            1,
            0,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE) - 1,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_PDF_FILE,
            "r",
            0,
            2,
            0,
        ),
    ],
)
def test_seek_file(file_storage, file_path, mode, location, whence, expected_size):
    seek_position = file_storage.seek(file_path, location, whence)
    file_contents = file_storage.read(
        path=file_path, mode=mode, seek_position=seek_position
    )
    print(file_contents)
    assert len(file_contents) == expected_size


@pytest.mark.parametrize("provider", [(FileStorageProvider.GCS)])
def test_file(provider):
    os.environ.clear()
    file_storage = FileStorage(provider=provider)
    assert file_storage is not None
    with pytest.raises(FileOperationError):
        file_storage.exists(TEST_CONSTANTS.READ_PDF_FILE)
    load_dotenv()


@pytest.mark.parametrize(
    "file_storage, lpath, rpath, recursive, expected_result",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER,
            True,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER,
            True,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_FOLDER_PATH,
            TEST_CONSTANTS.TEST_FOLDER,
            True,
            True,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_FOLDER_PATH,
            TEST_CONSTANTS.TEST_FOLDER,
            False,
            False,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER,
            True,
            True,
        ),
    ],
)
def test_cp(file_storage, lpath, rpath, recursive, expected_result):
    file_storage.cp(lpath, rpath, recursive=recursive, overwrite=True)
    actual_result = file_storage.exists(rpath)
    assert actual_result == expected_result
    if actual_result:
        file_storage.rm(rpath, recursive=True)
    assert file_storage.exists(rpath) is False


def test_pdf_read():
    fs = file_storage(FileStorageProvider.LOCAL)
    pdf_contents = io.BytesIO(fs.read(path=TEST_CONSTANTS.READ_PDF_FILE, mode="rb"))
    page_count = 0
    with pdfplumber.open(pdf_contents) as pdf:
        # calculate the number of pages
        page_count = len(pdf.pages)
        print(f"Page count: {page_count}")
    assert page_count == 7


@pytest.mark.parametrize(
    "file_storage, path, expected_size",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            os.path.getsize(TEST_CONSTANTS.READ_TEXT_FILE),
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_PDF_FILE,
            os.path.getsize(TEST_CONSTANTS.READ_PDF_FILE),
        ),
    ],
)
def test_file_size(file_storage, path, expected_size):
    file_size = file_storage.size(path=path)
    assert file_size == expected_size


@pytest.mark.parametrize(
    "file_storage, path, read_length, expected_mime_type",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            FileOperationParams.READ_ENTIRE_LENGTH,
            MimeType.PDF,
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            FileOperationParams.READ_ENTIRE_LENGTH,
            MimeType.TEXT,
        ),
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_PDF_FILE,
            FileOperationParams.READ_ENTIRE_LENGTH,
            MimeType.PDF,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            50,
            MimeType.TEXT,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_PDF_FILE,
            100,
            MimeType.PDF,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            100,
            MimeType.TEXT,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_PDF_FILE,
            100,
            MimeType.PDF,
        ),
    ],
)
def test_file_mime_type(file_storage, path, read_length, expected_mime_type):
    mime_type = file_storage.mime_type(path=path)
    file_storage.mkdir(path=TEST_CONSTANTS.READ_FOLDER_PATH)
    assert mime_type == expected_mime_type
    mime_type_read_length = file_storage.mime_type(path=path, read_length=read_length)
    file_storage.mkdir(path=TEST_CONSTANTS.READ_FOLDER_PATH)
    assert mime_type_read_length == expected_mime_type


@pytest.mark.parametrize(
    "file_storage, from_path, to_path",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/1.txt",
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/2.txt",
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/3.txt",
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/3.txt",
        ),
    ],
)
def test_download(file_storage, from_path, to_path):
    local_file_storage = FileStorage(FileStorageProvider.LOCAL)
    if local_file_storage.exists(to_path):
        local_file_storage.rm(to_path, recursive=True)
    assert local_file_storage.exists(to_path) is False
    file_storage.download(from_path, to_path)
    assert local_file_storage.exists(to_path) is True


@pytest.mark.parametrize(
    "file_storage, from_path, to_path",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/1.txt",
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/2.txt",
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/3.txt",
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_TEXT_FILE,
            TEST_CONSTANTS.TEST_FOLDER + "/3.txt",
        ),
    ],
)
def test_upload(file_storage, from_path, to_path):
    local_file_storage = FileStorage(FileStorageProvider.LOCAL)
    assert local_file_storage.exists(from_path) is True
    if file_storage.exists(to_path):
        file_storage.rm(to_path, recursive=True)
    file_storage.upload(from_path, to_path)
    assert file_storage.exists(to_path) is True


@pytest.mark.parametrize(
    "file_storage, file_path, expected_result",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "4a08b5721f75657eb883202cae16c74ca62df2c605e4126e50f4bf341d4fd693",
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "4a08b5721f75657eb883202cae16c74ca62df2c605e4126e50f4bf341d4fd693",
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "4a08b5721f75657eb883202cae16c74ca62df2c605e4126e50f4bf341d4fd693",
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_TEXT_FILE,
            "4a08b5721f75657eb883202cae16c74ca62df2c605e4126e50f4bf341d4fd693",
        ),
    ],
)
def test_get_hash_from_file(file_storage, file_path, expected_result):
    actual_file_hash = file_storage.get_hash_from_file(file_path)
    assert actual_file_hash == expected_result


@pytest.mark.parametrize(
    "file_storage, folder_path, expected_result",
    [
        (
            file_storage(provider=FileStorageProvider.GCS),
            TEST_CONSTANTS.READ_FOLDER_PATH + "/*.pdf",
            1,
        ),
        (
            file_storage(provider=FileStorageProvider.LOCAL),
            TEST_CONSTANTS.READ_FOLDER_PATH + "/*.txt",
            2,
        ),
        (
            file_storage(provider=FileStorageProvider.MINIO),
            TEST_CONSTANTS.READ_FOLDER_PATH + "/*.pdf",
            1,
        ),
        (
            file_storage(provider=FileStorageProvider.AZURE),
            TEST_CONSTANTS.READ_FOLDER_PATH + "/*.pdf",
            1,
        ),
    ],
)
def test_glob(file_storage, folder_path, expected_result):
    file_list = file_storage.glob(path=folder_path)
    print(f"Files: {file_list}")
    assert len(file_list) == expected_result


@pytest.mark.parametrize(
    "storage_type, env_name, expected",
    [
        (
            StorageType.PERMANENT,
            "TEST_PERMANENT_STORAGE_GCS",
            FileStorageProvider.GCS,
        ),
        (
            StorageType.SHARED_TEMPORARY,
            "TEST_TEMPORARY_STORAGE",
            FileStorageProvider.MINIO,
        ),
        (
            StorageType.PERMANENT,
            "TEST_LOCAL_STORAGE",
            FileStorageProvider.LOCAL,
        ),
        (
            StorageType.PERMANENT,
            "TEST_PERMANENT_STORAGE_AZURE",
            FileStorageProvider.AZURE,
        ),
    ],
)
def test_get_storage(storage_type, env_name, expected):
    file_storage = EnvHelper.get_storage(storage_type, env_name)
    assert file_storage.provider == expected
    print(file_storage)


@pytest.mark.parametrize(
    "storage_type, env_name, path",
    [
        (
            StorageType.PERMANENT,
            "TEST_PERMANENT_STORAGE_GCS",
            "fsspec-test",
        ),
        (
            StorageType.SHARED_TEMPORARY,
            "TEST_TEMPORARY_STORAGE",
            "unstract/execution/mock_org/"
            "13484b52-2127-48c2-b1a3-b517365346c3/"
            "39fcdcba-90bb-44ce-9446-67253adcb4d7/COPY_TO_FOLDER",
        ),
    ],
)
def test_dir_walk(storage_type, env_name, path):
    file_storage = EnvHelper.get_storage(storage_type, env_name)
    try:
        root, dirs, files = next(file_storage.walk(path))
    except StopIteration:
        return []
    for dir_name in dirs:
        print(dir_name)
    for file_name in files:
        print(file_name)
    if storage_type == StorageType.PERMANENT:
        assert len(files) > 0
    elif storage_type == StorageType.SHARED_TEMPORARY:
        assert len(files) == 0


def list_print_dir(file_storage, path, iter_num):
    print(f"PATH: {path}")
    print(f"\nItertion: {iter_num}")
    try:
        root, dirs, files = next(file_storage.walk(path))
    except StopIteration:
        return []
    for dir_name in dirs:
        print(dir_name)
    for file_name in files:
        print(file_name)
    print(f"Files: {files}")


@pytest.mark.parametrize(
    "storage_type, env_name, path",
    [
        (
            StorageType.SHARED_TEMPORARY,
            "TEST_TEMPORARY_STORAGE",
            "unstract/execution/mock_org/"
            "13484b52-2127-48c2-b1a3-b517365346c3/b"
            "f7b3d81-d0aa-4e9e-883d-25dd0f3a6466/COPY_TO_FOLDER",
        ),
    ],
)
def test_dir_ls(storage_type, env_name, path):
    new_file = os.path.join(path, "tmp.txt")
    file_storage = EnvHelper.get_storage(storage_type, env_name)
    if file_storage.exists(new_file):
        file_storage.rm(new_file)
    list_print_dir(file_storage, path, "1")
    file_storage.write(new_file, "w", data="Hello")
    list_print_dir(file_storage, path, "2")



import logging
import os
from typing import Optional

from llama_index.core import (
    MockEmbedding,
    SimpleDirectoryReader,
    StorageContext,
    VectorStoreIndex,
)
from llama_index.core.llms import MockLLM
from llama_index.core.vector_stores.types import BasePydanticVectorStore

from unstract.sdk.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.exceptions import VectorDBError

logger = logging.getLogger(__name__)


class VectorDBHelper:
    @staticmethod
    def test_vector_db_instance(
        vector_store: Optional[BasePydanticVectorStore],
    ) -> bool:
        try:
            if vector_store is None:
                return False

            storage_context = StorageContext.from_defaults(vector_store=vector_store)
            local_path = f"{os.path.dirname(__file__)}/samples/"
            # Using mock llm and embedding here.
            # For custom embedding args will be:
            #     embed_model - InstructorEmbeddings(embed_batch_size=2)
            #     chunk_size - 512
            #     llm=None
            llm = MockLLM()
            embed_model = MockEmbedding(
                embed_dim=VectorDbConstants.DEFAULT_EMBEDDING_SIZE
            )
            index = VectorStoreIndex.from_documents(
                # By default, SimpleDirectoryReader discards paths which
                # contain one or more parts that are hidden.
                # In local, packages could be installed in a venv. This
                # means a path can contain a ".venv" in it which will
                # then be treated as hidden and subsequently discarded.
                documents=SimpleDirectoryReader(
                    local_path, exclude_hidden=False
                ).load_data(),
                storage_context=storage_context,
                llm=llm,
                embed_model=embed_model,
            )
            query_engine = index.as_query_engine(llm=llm)

            query_engine.query("What did the author learn?")
            return True

        except Exception as e:
            logger.error(f"Error occured while testing adapter {e}")
            raise VectorDBError(message=str(e), actual_err=e)

    @staticmethod
    def get_collection_name(
        collection_name_prefix: str,
        embedding_dimension: int,
    ) -> str:
        """
        Notes:
            This function constructs the collection / table name to store the
            documents in the vector db.
            If user supplies this field in the config metadata then system
            would pick that and append it as prefix to embedding type.
            If this does not come as user setting, then system looks for it
            in the get_vector_db() argument and append it to embedding type
            If it is not there in both places then system appends
            "unstract_vector_db" as prefix to embedding type.
            If embedding type is not passed in get_vector_db() as arg,
            then system ignores appending that
        Args:
            collection_name_prefix (str): the prefix to be added. If this is
                    not passed in, then the default DEFAULT_VECTOR_DB_NAME
                    will be picked up for prefixing
            embedding_dimension (str): this will be suffixed.
                    If this value is not passed in,
                    then only collection_name_prefix will be returned
                Eg. collection_name_prefix -> mock_org
                    embedding_dimension -> 1536
                    return value -> mock_org_unstract_1536

                    collection_name_prefix -> No value
                    embedding_type -> No value
                    return value -> unstract_vector_db

        """
        vector_db_collection_name: str = VectorDbConstants.DEFAULT_VECTOR_DB_NAME
        if embedding_dimension:
            vector_db_collection_name = (
                vector_db_collection_name + "_" + str(embedding_dimension)
            )
        if collection_name_prefix:
            vector_db_collection_name = (
                collection_name_prefix + "_" + vector_db_collection_name
            )
        logger.debug(f"Resolved vectorDB name: {vector_db_collection_name}")
        return vector_db_collection_name



from abc import ABC, abstractmethod
from datetime import datetime
from typing import Any, Union

from fsspec import AbstractFileSystem

from unstract.sdk.file_storage.constants import FileOperationParams, FileSeekPosition


class FileStorageInterface(ABC):
    @abstractmethod
    def read(
        self,
        path: str,
        mode: str,
        encoding: str = FileOperationParams.DEFAULT_ENCODING,
        seek_position: int = 0,
        length: int = FileOperationParams.READ_ENTIRE_LENGTH,
    ) -> Union[bytes, str]:
        pass

    @abstractmethod
    def write(
        self,
        path: str,
        mode: str,
        encoding: str = FileOperationParams.DEFAULT_ENCODING,
        seek_position: int = 0,
        data: Union[bytes, str] = "",
    ) -> int:
        pass

    @abstractmethod
    def seek(
        self,
        file_handle: Union[AbstractFileSystem],
        location: int = 0,
        position: FileSeekPosition = FileSeekPosition.START,
    ) -> int:
        pass

    @abstractmethod
    def mkdir(self, path: str, create_parents: bool):
        pass

    @abstractmethod
    def exists(self, path: str) -> bool:
        pass

    @abstractmethod
    def ls(self, path: str) -> list[str]:
        pass

    @abstractmethod
    def rm(self, path: str, recursive: bool = True):
        pass

    @abstractmethod
    def cp(
        self,
        lpath: str,
        rpath: str,
        recursive: bool = False,
        overwrite: bool = True,
    ):
        pass

    @abstractmethod
    def size(self, path: str) -> int:
        pass

    @abstractmethod
    def modification_time(self, path: str) -> datetime:
        pass

    @abstractmethod
    def mime_type(
        self,
        path: str,
        read_length: int = FileOperationParams.READ_ENTIRE_LENGTH,
    ) -> str:
        pass

    @abstractmethod
    def download(self, from_path: str, to_path: str):
        pass

    @abstractmethod
    def glob(self, path: str) -> list[str]:
        pass

    @abstractmethod
    def get_hash_from_file(self, path: str) -> str:
        pass

    @abstractmethod
    def json_dump(
        self,
        path: str,
        data: dict[str, Any],
        **kwargs: dict[Any, Any],
    ):
        pass

    @abstractmethod
    def yaml_dump(
        self,
        path: str,
        data: dict[str, Any],
        **kwargs: dict[Any, Any],
    ):
        pass

    @abstractmethod
    def json_load(self, path: str) -> dict[Any, Any]:
        pass

    @abstractmethod
    def yaml_load(
        self,
        path: str,
    ) -> dict[Any, Any]:
        pass

    @abstractmethod
    def guess_extension(self, path: str) -> str:
        pass

    @abstractmethod
    def walk(self, path: str):
        pass



class VectorDbConstants:
    VECTOR_DB_NAME = "collection_name"
    EMBEDDING_DIMENSION = "embedding_dimension"
    DEFAULT_VECTOR_DB_NAME = "unstract"
    DEFAULT_EMBEDDING_SIZE = 2
    WAIT_TIME = "wait_time"



from .llm_whisperer import LLMWhisperer

metadata = {
    "name": LLMWhisperer.__name__,
    "version": "1.0.0",
    "adapter": LLMWhisperer,
    "description": "LLMWhisperer X2Text adapter",
    "is_active": True,
}
